%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OM2LClam.oz ; This file is part of the MathWeb system
%
%   major updates: 19.4.2001
%
%   Authors: Juergen Zimmer
%
%   email: jzimmer@ags.uni-sb.de
%   
% For information about this program, write to:                          
%   MathWeb-Project                                                        
%   AG Siekmann/FB nformatik                                            
%   Universitaet des Saarlandes                                          
%   Bau 36, 4. Stock                                                     
%   D-66041 Saarbruecken                                                 
%   Germany    
%
% For information about the newest version of MathWeb, see 
%   http://www.mathweb.org/mathweb
%
% See the file "LICENSE" or
%    http://www.mathweb.org/mathweb/LICENSE.html
% for information on usage and redistribution
% of this file, and for a DISCLAIMER OF ALL
% WARRANTIES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%%  Functions for translation of OpenMath and OMDoc elements to Lambda_Clam syntax
%%  in Teyjus lambda prolog.
%%
functor

import
   Property
   System
   Browser(browse: Browse)
   
export
   om2LClam:        OM2LClam
   omdoc2LClam:     OMDoc2LClam
   
define
   % taken from lclam arithmetic.mod:
%has_otype arithmetic zero nat.
%has_otype arithmetic s (nat arrow nat).
%has_otype arithmetic plus ((tuple_type [nat, nat]) arrow nat).
%has_otype arithmetic minus ((tuple_type [nat, nat]) arrow nat).
%has_otype arithmetic otimes ((tuple_type [nat, nat]) arrow nat).
%has_otype arithmetic exp ((tuple_type [nat, nat]) arrow nat).
%has_otype arithmetic leq ((tuple_type [nat, nat]) arrow bool).
%has_otype arithmetic half (nat arrow nat).
%has_otype arithmetic double (nat arrow nat).
%has_otype arithmetic even (nat arrow bool).

   SymbolRecord = symbols('BASE': symbols('TRUE': 'TRUTH'
					  'FALSE': 'FALSEHOOD'
					  'FORALL': 'PI'
					  'EXISTS': 'SIGMA'
					 )
                          'arith1': symbols('times': otimes
                                            'divide': odiv
                                            'power': exp)
			  'logic1': symbols('implies': imp)
			  
                          'LCLAM': symbols('IN-IMAGE': '%'
                                          )
			 )
   
   InfixOpsRec = infix(%'OR': "OR"#true
                       %%'AND':"AND"#true
                       %'IMPLIES':"IMPLIES"#true
		      )
   InfixOps = {Record.toDictionary InfixOpsRec}
   
   {Property.put 'print.depth' 100}
   {Property.put 'print.width' 100}
   
   VS2A = VirtualString.toAtom
   VS2BS = ByteString.make
   
   TYPE      = 'OMS'(cd: "mltt"
                     name: "type")
   
   FUNTYPE   = 'OMS'(cd: "mltt"
                     name: "funtype")
   
   LAMBDA    = 'OMS'(cd:  "fns1"
                     name:  "lambda")

   INTERVAL  = 'OMS'(name: "integer_interval" cd: "interval1")

   ALL_TYPES = 'OMS'(cd: "mltt"
                     name:  "all-types")
   
   FORALL = 'OMS'(cd: "quant1"
                  name: "forall")
   
   IMPLIES = 'OMS'(cd: "logic1"
                   name: "implies")
   
   EQUIV = 'OMS'(cd: "logic1"
                 name: "equivalent")
   
   EQ = 'OMS'(cd: "relation1"
              name: "eq")

   ONE_HALF = 'OMA'('OMS'(name: "divide" cd: "arith1")
                    ['OMI'(1) 'OMI'(2)])
   
   fun{List2LClam Xs Skolemize Split}
      case Xs
      of nil then "nil"
      [] _|_ then
	 "["
	 #{RecList2LClam Xs ", " Skolemize Split}
	 #"]"
      end
   end

   fun{RecList2LClam Term Seperator Skolemize Split}
      case Term
      of nil then nil
      [] T|Tr then
         {OM2LClam T Skolemize Split}
	 #if Tr == nil then nil
	  else 
	     Seperator
	     #{RecList2LClam Tr Seperator Skolemize Split}
	  end
      else Term end
   end
   
   fun {FindType AList}
      case AList of nil then false
      [] (Key#Val)|Rest then
         case Key
         of !TYPE then Val
         else {FindType Rest}
         end
      end
   end
   
   fun {Lambda2LClam OMBVARs Quant Term Skolemize Split}
      case OMBVARs
      of nil then
         {OM2LClam Term Skolemize Split}
      [] OMV|Rest then
         if Quant == unit then nil
         else "(app "#Quant#" (tuple [nat, " end
         #"(abs "
         #{OM2LClam OMV Skolemize Split}
         #"\\ " 
         #{Lambda2LClam Rest Quant Term Skolemize Split}
         #")"
         #if Quant == unit then nil
          else "]))" end
         %% actually we should use FindType... ToDo
      end
   end

   fun {ReplaceUnderscore Name}
      case Name
      of &_|Rest then
         case {Reverse Rest}
         of &_|Core then
            "<"#{Map {Reverse Core} fun{$ C}
                                       case C
                                       of &_ then &-
                                       else C end
                                    end}
            #">"
         else Name end
      else
         {Map Name fun{$ C}
                      case C
                      of &_ then &-
                      else C end
                   end}
      end
   end

   % {OM2LClam OMObj ?Type BoundVars}
   fun{OM2LClam OMObj Skolemize Split}
%      {Browse oM2LClam(OMObj)}
      case OMObj
      of nil then "nil"
      [] _|_ then
         {List2LClam OMObj Skolemize Split}
      [] 'OMOBJ'(Obj) then
         {OM2LClam Obj Skolemize Split}
      [] 'OMA'('OMS'(name: "list" cd: "list1") Args) then
         {OM2LClam Args Skolemize Split}
      [] 'OMS'(name: Name cd: CD) then
         NameA = {VS2A Name}
         CDA = {VS2A CD}
      in
	 if {HasFeature SymbolRecord CDA}
	    andthen {HasFeature SymbolRecord.CDA NameA} then
	    (SymbolRecord.CDA).NameA
	 else
            NameA
         end
      [] 'OMV'(name: Name) then
         NewName = case Skolemize
                   of false then
                      Name
                   [] yes(exists: Existdict forall: Alldict sign: Sign) then
                      {Dictionary.condGet Alldict {VS2A Name} Name}
                   else Name end
      in
         {ReplaceUnderscore NewName}
%      [] !ONE_HALF then
%         "onehalf"
      [] 'OMA'(!INTERVAL Args) then
         {RecList2LClam Args ", " Skolemize Split}
         
      [] 'OMA'(!FORALL 'OMBIND'(OMS 'OMBVAR'(OMBVARs) Term)|_ ...) then
         {System.showInfo forall_case1}
         case Skolemize
         of false then {Lambda2LClam OMBVARs forall Term Skolemize Split}
         [] yes(exists: Existdict forall: Alldict sign: Sign) then
            {ForAll OMBVARs proc{$ Var}
                               Name = {OM2LClam Var false false}
                            in
                               case Sign
                               of positive then
                                  {Dictionary.put Alldict {VS2A Name} {Map Name Char.toLower}}
                               else
                                  {Dictionary.put Alldict {VS2A Name} {Map Name Char.toUpper}}
                               end
                            end}
%            {Browse binding({Dictionary.entries Alldict})}
            {OM2LClam Term Skolemize Split}
         else {Lambda2LClam OMBVARs forall Term Skolemize Split} end
      [] 'OMA'(!EQUIV [LHS RHS]) then
         if Split \= false then
            "equiv "#Split#" "
            #{OM2LClam LHS Skolemize false}
            #" "
            #{OM2LClam RHS Skolemize false}
         else
            "(app equiv "
            #"(tuple ["
            #{RecList2LClam [LHS RHS] ", " Skolemize Split}
            #"]))"
         end
      [] 'OMA'(!EQ [LHS RHS]) then
         if Split \= false then
            "equiv "#Split#" "#{OM2LClam LHS Skolemize false}
            #" "
            #{OM2LClam RHS Skolemize false}
         else
            "(app eq "
            #"(tuple ["
            #{RecList2LClam [LHS RHS] ", " Skolemize Split}
            #"]))"
         end            
      [] 'OMA'(Functor Args...) then
	 Func = {OM2LClam Functor Skolemize Split}
      in
         if {Length Args} == 1 then
            [A] = Args
         in
            "(app "
            #Func
            #" "
            #{OM2LClam A Skolemize Split}
            #")"
         else
            "(app "
            #Func
            #" (tuple ["
            #if Func == forall then "nat, " else nil end
            #{RecList2LClam Args ", " Skolemize Split}
            #"]))"
         end
      [] 'OMI'(I) then
         if I < 0 then
            "(app minus (tuple [zero, (app nat_to_real "#{Number2LClam {Abs I}}#")]))"
         else
            "(app nat_to_real "#{Number2LClam {Abs I}}#")"
         end
      [] 'OMATTR'('OMATP'(!TYPE#Type)|_
                  Variable) then 
         {OM2LClam Variable Skolemize Split}   %%ToDo proper handling of types
         
      [] 'OMBIND'(OMS OMBVARs Term) then
         case OMS of !LAMBDA then
            {Lambda2LClam OMBVARs unit Term Skolemize Split}
         [] !ALL_TYPES then
            %% skip all-types/kappa for now
            {OM2LClam Term Skolemize Split}
         [] !FORALL then
            {Lambda2LClam OMBVARs forall Term Skolemize Split}
         else
            OMObj 
         end
      elseif {IsVirtualString OMObj} then
         {VS2A OMObj}
      else
         OMObj
      end
   end

   fun{Number2LClam Number}
      case Number
      of 0 then "zero"
      [] N then
         "(app s "#{Number2LClam N-1}#")"
      end
   end
   
   %returns skolem(forall: ["x":"x"...] exists: ["y":"Y" ...] matrix: 'OMA'(...))
   fun{Lemma2LClam OMObj}
%      {Browse lemma2Clam(OMObj)}
      case OMObj
      of 'OMOBJ'(Obj) then
         {Lemma2LClam Obj}
      [] 'FMP'(assumptions: Ass conclusion: Concl) then
         Cond 
         case Ass of nil then Cond = "trueP"
         [] A|_ then          Cond = {OM2LClam A.omobj
                                      yes(exists: {NewDictionary}
                                          forall: {NewDictionary}
                                          sign: negative)
                                      false}
         else                 Cond = "trueP" end
         ConclFormula = {OM2LClam Concl.omobj  yes(exists: {NewDictionary}
                                                   forall: {NewDictionary}
                                                   sign: negative)
                         Cond
                        }
      in
         ConclFormula
      else
         {Browse lemmaUnknown(OMObj)}
         nil
      end
   end

   fun{Def2LClam OMObj}
%      {Browse def2Clam(OMObj)}
      case OMObj
      of 'OMOBJ'(Obj) then
         {Def2LClam Obj}
      else
         {Browse defUnknown(OMObj)}
         nil
      end
   end


   fun{Conj2LClam OMObj}
      case OMObj
      of 'OMOBJ'(Obj) then
         {Conj2LClam Obj}
      [] 'FMP'(assumptions: Ass conclusion: Concl) then
         Hyps = {OM2LClam {Map Ass fun{$ A}
                                      A.omobj
                                   end}
                 yes(exists: {NewDictionary}
                     forall: {NewDictionary}
                     sign: negative)
                 false}
         Kons = {OM2LClam Concl.omobj false false}
      in
         'sequent'(hyps: Hyps
                   concl: Kons)
      end
   end
   
   fun{OMDoc2LClam Obj}
      case Obj
      of 'omdoc'(items: Items ...) then
         {Map Items OMDoc2LClam}
      [] 'assertion'(type: Type id: NameString fmp: FMP ...) then
         Name = {Map NameString Char.toLower}
      in
         if Type == "axiom" orelse Type == "lemma" then
            Formula = {Lemma2LClam FMP}
         in
            lemma(name: Name
                  lclam: Formula)
         elsecase Type
         of "definition" then
            Formula = {Def2LClam FMP}
         in
            'definition'(name: Name
                         lclam: Formula)
         [] "conjecture" then
            Sequent = {Conj2LClam FMP}
         in
            'conjecture'(name: Name
                         lclam: Sequent)
% add_lemma arithmetic plus1 both trueP (app plus (tuple [zero, Y])) Y.
         else
            {Browse unknown(Obj)}
            nil
         end
      else
         {Browse unknown(Obj)}
         nil
      end
   end

end %define

% app forall (tuple [nat, (abs x\ (app eq (tuple [(app plus (tuple [(app abs (tuple [x]))])), (app minus (tuple [x, y]))])))]))

%          <OMATTR>	 
%   	         <OMATP>
%                 <OMS cd="mltt" name="type" />
%                 <OMA>
%                   <OMS cd="mltt" name="funtype" />
%                   <OMV name="AA" />
%                   <OMA>
%                     <OMS cd="mltt" name="funtype" />
%                     <OMV name="AA" />
%                     <OMS cd="GENERIC" name="O" />
%                   </OMA>
%                 </OMA>
%               </OMATP>
%               <OMV name="R" />
%           </OMATTR>
