%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OpenMath.oz ; This file is part of the MathWeb system
%
%   major updates: 14.1.2002
%
%   Authors: Juergen Zimmer
%
%   email: jzimmer@ags.uni-sb.de
%   
% For information about this program, write to:                          
%   MathWeb-Project                                                        
%   AG Siekmann/FB nformatik                                            
%   Universitaet des Saarlandes                                          
%   Bau 36, 4. Stock                                                     
%   D-66041 Saarbruecken                                                 
%   Germany    
%
% For information about the newest version of MathWeb, see 
%   http://www.mathweb.org/mathweb
%
% See the file "LICENSE" or
%    http://www.mathweb.org/mathweb/LICENSE.html
% for information on usage and redistribution
% of this file, and for a DISCLAIMER OF ALL
% WARRANTIES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%
%% Useful functions for OpenMath handling
%%

functor

import
   Error
   Property
   System
   Browser(browse: Browse)
   
export
   om2Maple: OM2Maple
   
define
   {Property.put 'print.depth' 100}
   {Property.put 'print.width' 100}
   
   VS2A = VirtualString.toAtom
   VS2S = VirtualString.toString
   VS2BS = ByteString.make
   
   InfixOpsRec = infix('plus': "+"#true
                       'minus':"-"#true
                       'times':"*"#true
                       'divide':"/"#true
                       'power':"^"#true
                       'leq':  "<="#true
                       'geq':  ">="#true
                       'lt':   "<"#true
                       'gt':   ">"#true
                       'eq':   "="#false
		       'funtype': "->"#true
		      )
   InfixOps = {Record.toDictionary InfixOpsRec}
   
   FuncDict = {NewDictionary}

   TYPE      = 'OMS'(cd: {VS2BS "mltt"}
		     name: {VS2BS "type"})
   
   LAMBDA    = 'OMS'(cd: {VS2BS "fns1"}
                     name: {VS2BS "lambda"})
   
   ALL_TYPES = 'OMS'(cd: {VS2BS "mltt"}
                     name: {VS2BS "all-types"})

   fun {FindType AList}
      case AList of nil then false
      [] (Key#Val)|Rest then
         case Key
         of !TYPE then Val
         else {FindType Rest}
         end
      end
   end

   fun {Var2Maple OMV}
      case OMV
      of 'OMATTR'(AList Var) then
	 Var.name
	 /*
	 case {FindType AList}
	 of false then
	    Var.name
	 [] Type then
	    "("#Var.name#" "#{WriteType Type}#")"
	 end
	 */
      else
         OMV.name
      end
   end
   
   fun {Lambda2Maple OMBVARs Term}
      case OMBVARs of nil then Term
      [] OMV|Rest then
	 "lam "#{Var2Maple OMV}#" . "#{Lambda2Maple Rest Term}
      end
   end
   
   fun{List2Maple Xs  FuncDict}
      case Xs
      of nil then "NIL"
      [] _|_ then
	 "("
	 #{RecList2Maple Xs  FuncDict}
	 #")"
      end
   end
   
   fun{RecList2Maple Term  FuncDict}
      case Term
      of nil then nil
      [] T|Tr then
	 {OM2Maple T}
	 #if Tr == nil then ""
	  else 
	     " , "
	     #{RecList2Maple Tr FuncDict}
	  end
      else Term end
   end
   
   fun{OM2Maple OMObj}
%      {Browse om2Maple(OMObj)}
      case OMObj
      of nil then ""
      [] _|_ then
         {List2Maple OMObj FuncDict}
      [] 'OMOBJ'(Obj) then
         {OM2Maple Obj}
      [] 'OMA'('OMS'(name: "sum"  cd: "arith1")
               ['OMA'('OMS'(name: "integer_interval" cd: "interval1")
                      [Lower Upper])
                'OMBIND'(LAMBDA [X] Body)])
      then 
         "sum("#{OM2Maple Body}#", "
         #{OM2Maple X}#"="#{OM2Maple Lower}#".."#{OM2Maple Upper}
         #")"
      [] 'OMA'('OMS'(name: "list" cd: "list1") Args...) then
	 {OM2Maple Args}
      [] 'OMS'(name: Name cd: CD...) then
         {VS2A Name}
      [] 'OMV'(name: Name...) then
         {VS2A Name}
      [] 'OMA'(Functor Args...) then
	 Func = {OM2Maple Functor}
      in
	 case if {IsAtom Func} then
		 {Dictionary.condGet InfixOps Func unit}
	      else unit end
         of unit then 
            Func
            #"("
	    #{RecList2Maple Args  FuncDict}
            #")"
         [] Op#Parans then
            case Args
            of [A1 A2] then                
	       if Parans then "(" else "" end
               #{OM2Maple A1}
	       #Op
               #{OM2Maple A2}
	       #if Parans then ")" else "" end
	    else
	       Func
	       #"("
	       #{RecList2Maple Args  FuncDict}
	       #")"
            end
         end
      [] 'OMI'(I...) then
         I
      [] 'OMF'(F...) then
	 {Float.toString F}
      [] 'OMBIND'(OMS OMBVARs Term) then
	 case OMS of !LAMBDA then
	    {Lambda2Maple OMBVARs {OM2Maple Term}}
	 [] !ALL_TYPES then
	    %% skip all-types/kappa for now
	    {OM2Maple Term}
	 else
	    {OM2Maple OMS}
	    #{OM2Maple OMBVARs}
	    #". "
	    #{OM2Maple Term}
	 end
      [] 'OMATTR'(AList Term) then
	 {Browse {List.toRecord 'attr'
		  {Map AList
		   fun {$ X#Y}
		      {VS2A X.cd#":"#X.name}#{VS2BS {OM2Maple Y}}
		   end}}}
	 {OM2Maple Term}
      elseif {IsVirtualString OMObj} then
         {VS2A OMObj}
      else
	 {Browse OMObj}
         "_" %{VS2A OMObj}
      end
   end

          
end %define
   

