%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OM2Post.oz ; This file is part of the MathWeb system
%
%   major updates: 19.4.2001
%
%   Authors: Juergen Zimmer
%
%   email: jzimmer@ags.uni-sb.de
%   
% For information about this program, write to:                          
%   MathWeb-Project                                                        
%   AG Siekmann/FB nformatik                                            
%   Universitaet des Saarlandes                                          
%   Bau 36, 4. Stock                                                     
%   D-66041 Saarbruecken                                                 
%   Germany    
%
% For information about the newest version of MathWeb, see 
%   http://www.mathweb.org/mathweb
%
% See the file "LICENSE" or
%    http://www.mathweb.org/mathweb/LICENSE.html
% for information on usage and redistribution
% of this file, and for a DISCLAIMER OF ALL
% WARRANTIES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Useful functions for OpenMath handling
%%
functor

import
   Error
   System
   Property
   Browser(browse: Browse)
   
%   Internet     at 'x-ozlib://mathweb/share/Internet.ozf'

   %% not used anymore:
   %%MBaseShare(cmpItemsToVS: OMObjs2Post)
   %%at 'x-ozlib://mathweb/mbase/export/html/Share.ozf'
   
export
   om2Post:        OM2Post
   om2TermPrinter: OM2TermPrinter
   
define
   /* Lisp stuff:
   (setf omcont*symbol-cd-mapping 
     ((PLUS  . ("plus" "arith1"))
     (MINUS . ("minus" "arith1"))
     (TIMES . ("times" "arith1"))
     (POWER . ("power" "arith1"))
     (ABSVAL . ("abs" "arith1"))
     
     (=        . ("eq" "relation1"))
     (LESS     . ("lt" "relation1"))
     (GREATER  . ("gt" "relation1"))
    )
   )
   */
   SymbolRecord = symbols('arith1': symbols('abs': "absval"
                                            'unary_minus': "change-sign"
                                           )
                          'relation1': symbols('eq': "="
                                               'lt': "less"
                                               'gt': "greater"
                                              )
                         )

   VS2A = VirtualString.toAtom
   VS2S = VirtualString.toString
   VS2BS = ByteString.make

   TYPE      = 'OMS'(cd:  {VS2BS "mltt"}
                     name:  {VS2BS "type"})
   
   LAMBDA    = 'OMS'(cd:  {VS2BS "fns1"}
                     name:   {VS2BS "lambda"})
   
   ALL_TYPES = 'OMS'(cd:  {VS2BS "mltt"}
                     name:  {VS2BS "all-types"})

   FUNTYPE   = 'OMS'(cd:  {VS2BS "mltt"}
                     name:  {VS2BS "funtype"})
   
   
   {Property.put 'print.depth' 100}
   {Property.put 'print.width' 100}
   
   
   InfixOpsRec = infix('plus':"+"
                       'minus':"-"
                       'times':"*"
                       'power':"^"
                      )
   InfixOps = {Record.toDictionary InfixOpsRec}
   
   fun {ExceptionToVS ERROR}
      {Error.messageToVirtualString {Error.exceptionToMessage ERROR}}
   end
   
   fun{List2Post Xs}
      case Xs
      of nil then "NIL"
      [] _|_ then
	 "("
	 #{RecList2Post Xs}
	 #")"
      end
   end
   
   fun{RecList2Post Term}
      case Term
      of nil then nil
      [] T|Tr then
         {OM2Post T}
	 #if Tr == nil then ""
	  else 
	     " "
	     #{RecList2Post Tr}
	  end
      else Term end
   end
   
   
   fun {FindType AList}
      case AList of nil then false
      [] (Key#Val)|Rest then
         case Key
         of !TYPE then Val
         else {FindType Rest}
         end
      end
   end

   fun {WriteType Type}
      "todo"
   end
   
   fun {Var2Post OMV}
      case OMV
      of 'OMATTR'(AList Var) then
         case {FindType AList}
         of false then
            Var.name
         [] Type then
            "("#Var.name#" "#{WriteType Type}#")"
         end
      else
         OMV.name
      end
   end   
   
   fun {Lambda2Post OMBVARs Term}
      case OMBVARs of nil then Term
      [] OMV|Rest then
         "(lam "#{Var2Post OMV}#" "#{Lambda2Post Rest Term}#")"
      end
   end
   
   fun{OM2Post OMObj}
%      {Browse om2Post(OMObj)}
      case OMObj
      of nil then ""
      [] _|_ then
         {List2Post {Map OMObj OM2Post}}
      [] 'OMOBJ'(Obj) then
         {OM2Post Obj}
      [] 'OMA'('OMS'(name: "list" cd: "list1") Args) then
         {OM2Post Args}
 
      [] 'OMA'(!FUNTYPE [Domain Codomain]) then 
         "("
         #{OM2Post Codomain}
         #" "
         #{OM2Post Domain}
         #") "
      [] 'OMS'(name: Name cd: CD) then
         NameA = {VS2A Name}
         CDA = {VS2A CD}
      in
         if {HasFeature SymbolRecord CDA} andthen {HasFeature SymbolRecord.CDA NameA} then 
            (SymbolRecord.CDA).NameA
         else
            NameA
         end
      [] 'OMV'(name: Name) then
         {VS2A Name}
      [] 'OMA'(Functor Args) then
         %%{Browse 'OMA'('functor':Functor args:Args)}
         "("#{OM2Post Functor}
         #" "
         #if {IsList Args}
          then {RecList2Post Args}
          else {RecList2Post {Record.toList Args}}
          end
         #")"
      [] 'OMI'(I) then
         I
      [] 'OMBIND'(OMS OMBVARs Term) then
         case OMS of !LAMBDA then
            {Lambda2Post OMBVARs {OM2Post Term}}
         [] !ALL_TYPES then
            %% skip all-types/kappa for now
            {OM2Post Term}
         else
            raise OMObj end
         end
      elseif {IsVirtualString OMObj} then
         {VS2A OMObj}
      else
         raise OMObj end
      end
   end

fun{List2TermPrinter Xs}
   case Xs
   of nil then "NIL"
   [] _|_ then
	 "("
	 #{RecList2TermPrinter Xs}
	 #")"
      end
   end
   
   fun{RecList2TermPrinter Term}
      case Term
      of nil then nil
      [] T|Tr then
	 {OM2TermPrinter T}
	 #if Tr == nil then ""
	  else 
	     " "
	     #{RecList2TermPrinter Tr}
	  end
      else Term end
   end

   fun {Var2TermPrinter OMV}
      case OMV
      of 'OMATTR'(AList Var) then
         case {FindType AList}
         of false then
            Var.name
         [] Type then
            "["#Var.name#" "#{WriteType Type}#"]"
         end
      else
         OMV.name
      end
   end
   
   fun {Lambda2TermPrinter OMBVARs Term}
      case OMBVARs of nil then Term
      [] OMV|Rest then
         "(lam "#{Var2TermPrinter OMV}#" "#{Lambda2TermPrinter Rest Term}#")"
      end
   end
      
   fun{OM2TermPrinter OMObj}
%      {Browse om2TermPrinter(OMObj)}
      case OMObj
      of nil then ""
      [] _|_ then
         {List2TermPrinter {Map OMObj OM2TermPrinter}}
      [] 'OMOBJ'(Obj) then
         {OM2TermPrinter Obj}
      [] 'OMA'('OMS'(name: "list" cd: "list1") Args) then
         {OM2TermPrinter Args}
      [] 'OMS'(name: Name cd: CD) then
         NameA = {VS2A Name}
         CDA = {VS2A CD}
      in
         if {HasFeature SymbolRecord CDA} andthen {HasFeature SymbolRecord.CDA NameA} then 
            "["#(SymbolRecord.CDA).NameA#" dummy]"
         else
            "["#NameA#" i]"
         end
      [] 'OMV'(name: Name) then
         "["#{VS2A Name}#" i]"
      [] 'OMI'(I) then
         "["#I#" num]"
      [] 'OMF'(F) then
         Num = if ({Ceil F} == F) then
                  {FloatToInt F}
               else F end
      in
         "["#Num#" num]"
      [] 'OMA'(Functor Args) then
         "("#{OM2TermPrinter Functor}
         #" "
         #if {IsList Args}
          then {RecList2TermPrinter Args}
          else {RecList2TermPrinter {Record.toList Args}}
          end
         #")"
      [] 'OMBIND'(OMS OMBVARs Term) then
         case OMS of !LAMBDA then
            {Lambda2TermPrinter OMBVARs {OM2TermPrinter Term}}
         [] !ALL_TYPES then
            %% skip all-types/kappa for now
            {OM2TermPrinter Term}
         else
            "("#{OM2TermPrinter OMS}#
            " "#{Lambda2TermPrinter OMBVARs {OM2TermPrinter Term}}#")"
         end
         /* previous version (for reference):
      [] 'OMBIND'(OMS OMBVARs Term) then
         "("#{OM2TermPrinter OMS}
         #" (lam "
         #{OM2TermPrinter OMBVARs}
         #" "      %% ToDo: Guess the right type. This is non-trivial!
         #{OM2TermPrinter Term}
         #"))"
         */
      [] 'OMBVAR'(Var) then
         {OM2TermPrinter Var}
      end
   end
   

          
end %define
   



