%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OM2Pvs.oz ; This file is part of the MathWeb system
%
%   major updates: 19.4.2001
%
%   Authors: Juergen Zimmer
%
%   email: jzimmer@ags.uni-sb.de
%   
% For information about this program, write to:                          
%   MathWeb-Project                                                        
%   AG Siekmann/FB nformatik                                            
%   Universitaet des Saarlandes                                          
%   Bau 36, 4. Stock                                                     
%   D-66041 Saarbruecken                                                 
%   Germany    
%
% For information about the newest version of MathWeb, see 
%   http://www.mathweb.org/mathweb
%
% See the file "LICENSE" or
%    http://www.mathweb.org/mathweb/LICENSE.html
% for information on usage and redistribution
% of this file, and for a DISCLAIMER OF ALL
% WARRANTIES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Useful functions for OpenMath handling
%%
functor

import
   Property
%   System
   Browser(browse: Browse)
   
export
   om2Pvs:        OM2Pvs
   
define
   SymbolRecord = symbols('base': symbols('forall': 'forall'  %dummy
					 )
                          'integer': symbols('div': '/'
                                             'times': '*'
                                             'plus': '+'
                                             'minus': '-'
                                             
                                            )
                          'arith1': symbols(
                                            'div': '/'
                                            'times': '*'
                                            'plus': '+'
                                            'minus': '-'
                                           )
                          'relation1': symbols('leq': '<='
                                               'geq': '>='
                                              )
                          'natural': symbols('num': 'real'
                                            )
                          'generic': symbols('o': 'bool'
                                            )
			 )
   
   InfixOpsRec = infix('OR':      "OR"#false
                       'AND':     "AND"#false
                       'IMPLIES': "IMPLIES"#false
                       'or':      "OR"#false
                       'and':     "AND"#false
                       'implies': "IMPLIES"#false
                       '/': '/'#true
                       '*': '*'#true
                       '+': '+'#true
                       '-': '-'#true
                       '<=': '<='#false
                       '>=': '>='#false
                      )
   InfixOps = {Record.toDictionary InfixOpsRec}

   VariableRec = vars('NUM': "real")
   
   VariableRenaming =  {Record.toDictionary VariableRec}
   {Property.put 'print.depth' 100}
   {Property.put 'print.width' 100}
   
   VS2A = VirtualString.toAtom
   VS2BS = ByteString.make
   VS2S = VirtualString.toString
   
   TYPE      = 'OMS'(cd:  "mltt"
                     name:  "type")
   
   FUNTYPE   = 'OMS'(cd:  "mltt"
                     name:  "funtype")
   
   LAMBDA    = 'OMS'(cd:  "mltt"
                     name:  "lambda")
   
   ALL_TYPES = 'OMS'(cd:  "mltt"
                     name:  "all-types")
   
   FOR_ALL_OMEGA = 'OMS'(cd:  "BASE"
                         name:  "FORALL")
   
   FOR_ALL       = 'OMS'(cd: "quant1"
                         name: "forall")
   
   UNARY_MINUS   = 'OMS'(cd: "arith1"
                       name: "unary_minus")
   
   NOT           = 'OMS'(cd: "logic1"
                         name: "not")
   
   fun{List2Pvs Xs}
      case Xs
      of nil then "NIL"
      [] _|_ then
	 "("
	 #{RecList2Pvs Xs " "}
	 #")"
      end
   end
   
   fun{RecList2Pvs Term Seperator}
      case Term
      of nil then nil
      [] T|Tr then
         {OM2Pvs T}
	 #if Tr == nil then ""
	  else 
	     Seperator
	     #{RecList2Pvs Tr Seperator}
	  end
      else Term end
   end
   
   fun {FindType AList}
      case AList of nil then false
      [] (Key#Val)|Rest then
         case Key
         of !TYPE then Val
         else {FindType Rest}
         end
      end
   end

   fun {Lambda2Pvs OMBVars Term}
      case OMBVars of nil then
         ": "
         #{OM2Pvs Term}
      [] OMBVar|Rest then
         "LAMBDA "
         #{OM2Pvs OMBVar}
         #{Lambda2Pvs Rest Term}
         #" "
         %% actually we should use FindType... ToDo
      end
   end

% example:  LAMBDA P(OA) LAMBDA R(OA) LAMBDA z(A) [[P z] OR [R z]]

   fun{OM2Pvs OMObj}
%      {Browse oM2Pvs(OMObj)}
      case OMObj
      of nil then ""
      [] _|_ then
         {List2Pvs OMObj}
      [] 'OMOBJ'(Obj) then
         {OM2Pvs Obj}
      [] 'OMA'('OMS'(name: "list" cd: "list1") Args) then
         {OM2Pvs Args}
      [] 'OMI'(I) then
         I
      [] 'OMS'(name: Name cd: CD) then
         NameA = {VS2A Name}
         CDA = {VS2A CD}
      in
         if {HasFeature SymbolRecord CDA} andthen {HasFeature SymbolRecord.CDA NameA} then 
            (SymbolRecord.CDA).NameA
         else
            {Map {VS2S NameA} Char.toLower}
         end
      [] 'OMV'(name: Name) then
         VarA = {VS2A Name}
      in
         {Dictionary.condGet VariableRenaming VarA {Map {VS2S VarA} Char.toLower}}

%%%         <OMA><OMS cd="mltt" name="funtype"/><OMV name="num"/><OMA><OMS cd="mltt" name="funtype"/><OMV name="num"/>
%%% should become          [real -> real]         
         
%%%         <OMA><OMS cd="mltt" name="funtype"/><OMA><OMS cd="mltt" name="funtype"/><OMS cd="NATURAL" name="NUM"/><OMS cd="NATURAL" name="NUM"/></OMA><OMA><OMS cd="mltt" name="funtype"/><OMS cd="NATURAL" name="NUM"/><OMS cd="GENERIC" name="O"/><
%%% should become         [[real -> real] -> [real -> bool]]
         
      [] 'OMA'(!FUNTYPE Args) then
         case Args
         of A|Ar then
            "["
            #{OM2Pvs A}
            #" -> "
            #{RecList2Pvs Ar " "}
         else
            raise OMObj end
         end
      [] 'OMA'(!FOR_ALL_OMEGA OMBinding Term) then  %% handle the 'strange' OMEGA forall
         "FORALL "
         #{OM2Pvs OMBinding}          %% ToDo: skip the Lambda binding.
         #" : "
         #{OM2Pvs Term}
      [] 'OMA'(!UNARY_MINUS Args) then
         "-"
         #{RecList2Pvs Args nil}
      [] 'OMA'(!NOT Args) then
         "NOT "
         #{RecList2Pvs Args nil}
      [] 'OMA'(Functor Args...) then   
         Func = {OM2Pvs Functor}
%         {Browse func(Func)}
      in
	 case {Dictionary.condGet InfixOps {VS2A Func} unit}
         of unit then 
            {Map {VS2S Func} Char.toLower}
            #"("
	    #{RecList2Pvs Args ", "}
            #")"
         [] Op#Parans then
%            {Browse Args}
            case Args
            of [A1 A2] then
	       if Parans then "(" else "" end
               #{OM2Pvs A1}
	       #" "#Op#" "
               #{OM2Pvs A2}
	       #if Parans then ")" else "" end
            else nil
            end
         end
      [] 'OMSTR'(S) then
         S
      [] 'OMATTR'('OMATP'((!TYPE#Type))|_ Variable) then  %% type attribution for variables
         "("
         #{OM2Pvs Variable}
         #" : "
         #{OM2Pvs Type}         
         #") "
         #" "
      [] 'OMBIND'(OMS OMBVars Term ...) then
%    should result in:  continuous(LAMBDA x : 1 / (abs(x) + 1), y)
%         {Browse oms(OMS case OMS of !FOR_ALL then true else false end )}
         case OMS            
         of !LAMBDA then                  %% the standard Lambda abstraction
            'OMBVAR'(Vars) = OMBVars
         in
            "("
            #{Lambda2Pvs Vars Term}
            #")"
         [] !FOR_ALL then                 %% the forall abstraction
            "FORALL "
            #{RecList2Pvs OMBVars ", "}
            #" : "
            #{OM2Pvs Term}
         [] !ALL_TYPES then               %% forall-typs abstraction of OMEGA
            {OM2Pvs Term}                %% skip all-types/kappa for now
         else
            raise ombind(OMObj) end
         end
      elseif {IsVirtualString OMObj} then
         {VS2A OMObj}
      else
         raise unhandled(OMObj) end
      end
   end
   
end %define
   



%          <OMATTR>	 
%   	         <OMATP>
%                 <OMS cd="mltt" name="type" />
%                 <OMA>
%                   <OMS cd="mltt" name="funtype" />
%                   <OMV name="AA" />
%                   <OMA>
%                     <OMS cd="mltt" name="funtype" />
%                     <OMV name="AA" />
%                     <OMS cd="GENERIC" name="O" />
%                   </OMA>
%                 </OMA>
%               </OMATP>
%               <OMV name="R" />
%           </OMATTR>
