%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OM2Tps.oz ; This file is part of the MathWeb system
%
%   major updates: 19.4.2001
%
%   Authors: Juergen Zimmer
%
%   email: jzimmer@ags.uni-sb.de
%   
% For information about this program, write to:                          
%   MathWeb-Project                                                        
%   AG Siekmann/FB nformatik                                            
%   Universitaet des Saarlandes                                          
%   Bau 36, 4. Stock                                                     
%   D-66041 Saarbruecken                                                 
%   Germany    
%
% For information about the newest version of MathWeb, see 
%   http://www.mathweb.org/mathweb
%
% See the file "LICENSE" or
%    http://www.mathweb.org/mathweb/LICENSE.html
% for information on usage and redistribution
% of this file, and for a DISCLAIMER OF ALL
% WARRANTIES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Translates OpenMath formulas in TPS format
%%
functor

import
   Property
%   System
%   Browser(browse: Browse)
   
export
   om2Tps:        OM2Tps
   
define
   SymbolRecord = symbols('BASE': symbols('TRUE': 'TRUTH'
					  'FALSE': 'FALSEHOOD'
                                          'FORALL': 'PI'
                                          'EXISTS': 'SIGMA'
					 )
                          'TPS': symbols('IN-IMAGE': '%'
                                        )
			 )
   
   InfixOpsRec = infix('OR': "OR"#true
                       'AND':"AND"#true
                       'IMPLIES':"IMPLIES"#true
		      )
   InfixOps = {Record.toDictionary InfixOpsRec}
   
   {Property.put 'print.depth' 100}
   {Property.put 'print.width' 100}
   
   VS2A = VirtualString.toAtom
   VS2BS = ByteString.make
   
   TYPE      = 'OMS'(cd: {VS2BS "mltt"}
                     name: {VS2BS "type"})
   
   FUNTYPE   = 'OMS'(cd: {VS2BS "mltt"}
                     name: {VS2BS "funtype"})
   
   LAMBDA    = 'OMS'(cd: {VS2BS "fns1"}
                     name: {VS2BS "lambda"})
   
   ALL_TYPES = 'OMS'(cd: {VS2BS "mltt"}
                     name: {VS2BS "all-types"})

   FOR_ALL = 'OMS'(cd: {VS2BS "FORALL"}
                   name: {VS2BS "BASE"})
   
   fun{List2Tps Xs}
      case Xs
      of nil then "NIL"
      [] _|_ then
	 "("
	 #{RecList2Tps Xs " "}
	 #")"
      end
   end
   
   fun{RecList2Tps Term Seperator}
      case Term
      of nil then nil
      [] T|Tr then
         {OM2Tps T}
	 #if Tr == nil then ""
	  else 
	     Seperator
	     #{RecList2Tps Tr Seperator}
	  end
      else Term end
   end
   
   fun {FindType AList}
      case AList of nil then false
      [] (Key#Val)|Rest then
         case Key
         of !TYPE then Val
         else {FindType Rest}
         end
      end
   end

   fun {Lambda2Tps OMBVARs Term}
      case OMBVARs of nil then {OM2Tps Term}
      [] OMV|Rest then
         "[LAMBDA"
         #{OM2Tps OMV}
         #{Lambda2Tps Rest Term}
         #"]"
         %% actually we should use FindType... ToDo
      end
   end

% example:  LAMBDA P(OA) LAMBDA R(OA) LAMBDA z(A) [[P z] OR [R z]]

   fun{OM2Tps OMObj}
%      {Browse oM2Tps(OMObj)}
      case OMObj
      of nil then ""
      [] _|_ then
         {List2Tps OMObj}
      [] 'OMOBJ'(Obj) then
         '\"'#{OM2Tps Obj}#'\"'
      [] 'OMA'('OMS'(name: "list" cd: "list1") Args) then
         {OM2Tps Args}
      [] 'OMS'(name: Name cd: CD) then
         NameA = {VS2A Name}
         CDA = {VS2A CD}
      in
         if {HasFeature SymbolRecord CDA} andthen {HasFeature SymbolRecord.CDA NameA} then 
            (SymbolRecord.CDA).NameA
         else
            NameA
         end
      [] 'OMV'(name: Name) then
         {VS2A Name}
      [] 'OMA'(!FUNTYPE Args) then
         {RecList2Tps {Reverse Args} nil}
%       [] 'OMA'('OMBIND'(!FOR_ALL OMBVARs Term) ...) then
      [] 'OMA'(Functor Args...) then
	 Func = {OM2Tps Functor}
      in
	 case if {IsAtom Func} then
		 {Dictionary.condGet InfixOps Func unit}
	      else unit end
         of unit then 
            "["
            #Func
            #" "
	    #{RecList2Tps Args " "}
            #"]"
         [] Op#Parans then
            case Args
            of [A1 A2] then                
	       if Parans then "[" else "" end
               #{OM2Tps A1}
	       #" "#Op#" "
               #{OM2Tps A2}
	       #if Parans then "]" else "" end
            else nil
            end
         end
      [] 'OMI'(I) then
         I
      [] 'OMATTR'((!TYPE#Type)|_
                  Variable) then 
         " "
         #{OM2Tps Variable} 
         #"("
         #{OM2Tps Type}
         #") "
         #" "
      [] 'OMBIND'(OMS OMBVARs Term) then
         case OMS of !LAMBDA then
            {Lambda2Tps OMBVARs Term}
         [] !ALL_TYPES then
            %% skip all-types/kappa for now
            {OM2Tps Term}
         else
            raise OMObj end
         end
      elseif {IsVirtualString OMObj} then
         {VS2A OMObj}
      else
         raise OMObj end
      end
   end
   
end %define
   



%          <OMATTR>	 
%   	         <OMATP>
%                 <OMS cd="mltt" name="type" />
%                 <OMA>
%                   <OMS cd="mltt" name="funtype" />
%                   <OMV name="AA" />
%                   <OMA>
%                     <OMS cd="mltt" name="funtype" />
%                     <OMV name="AA" />
%                     <OMS cd="GENERIC" name="O" />
%                   </OMA>
%                 </OMA>
%               </OMATP>
%               <OMV name="R" />
%           </OMATTR>
