%%
%% printer for openmath objects
%%

functor
   
import
   XmlPrinter('class': XML)
   at 'x-ozlib://mathweb/share/XmlPrinter.ozf'
   
export
   'class':     NewOMWriter

   read:        ReadObject
   write:       WriteObject
      
define
   
   %%
   %% write OpenMath objects
   %%
   local
      skip
   in
      class NewOMWriter from XML
                           
	 attr
            indent: false
            prefix: ''
	    tab:    '  '
	    
	 meth init(prefix: Prefix <= "\n"
		   tab:    Tab    <= "  "
                   indent: Indent <= true)
	    prefix <- Prefix
	    tab    <- Tab
            indent <- Indent
	 end
         %%
         meth indentOn
            indent <- true
         end
         meth indentOff
            indent <- false
         end
         %%
	 meth setPrefix(Prefix)
	    prefix <- Prefix
	 end
         %%
         meth setTab(N)
	    Tab = {List.make N} in
	    {ForAll Tab proc {$ C} C=&  end}
	    tab <- Tab
	 end
         meth pushTab(Tab<=@tab)
	    prefix <- @prefix#Tab
	 end
	 meth popTab
	    Prefix#_ = @prefix in
	    prefix <- Prefix
	 end
         %%
         meth writePrefix
            if @indent then {self write(vs:@prefix)} end
         end
         meth writePrefix1
            if @indent then {self write(vs:@prefix#@tab)} end
         end
         %%
         meth startTag(...) = StartTag
            {self writePrefix1}
            XML, StartTag
         end
         meth endTag(...) = EndTag
            {self writePrefix}
            XML, {Record.subtract EndTag indent} % workaround for bug
         end
         meth emptyTag(...) = EmptyTag
            {self writePrefix1}
            XML, EmptyTag
         end
         meth tag(T)
            {self writePrefix1}
            XML, startTag(T)
            {ForAll {Filter {Arity T} IsInt}
             proc {$ I}
                XML, tag(T.I)
             end}
            XML, endTag(T)
         end
         %%
         meth writeElems(L)
            {ForAll L
             proc {$ Elem}
                {self write(vs:@prefix#@tab)}
                {self writeElem(Elem)}
             end}
         end
         
         %% simple elements
	 meth writeOMS(X)
	    {self emptyTag(X)}  % 'OMS'(cd:CD name:Name)
	 end
	 meth writeOMV(X)
	    {self emptyTag(X)} % 'OMV'(name:Name)
	 end
	 meth writeOMI(X)
	    {self tag(X)} % 'OMI'(I)
	 end
	 meth writeOMB(X)
	    {self tag(X)} % 'OMB'(BS)
	 end
	 meth writeOMSTR(X)
	    {self tag(X)} % 'OMSTR'(BS)
	 end
	 meth writeOMF(X)
	    'OMF'(F) = X in
	    {self emptyTag('OMF'(dec:{Float.toString F}))}
	 end
         %% complex elements
	 meth writeOMA(X)
            case X of 'OMA'(F Args.../*'brackets-hint':_*/) then
               {self startTag('OMA')}
               {self write(vs:@prefix#@tab)}
               {self writeElem(F)}
               {self writeElems(Args)}
               {self write(vs:@prefix)}
               {self endTag('OMA')}
            [] 'OMA'(F|Args) then % hack for demo
               {self startTag('OMA')}
               {self write(vs:@prefix#@tab)}
               {self writeElem(F)}
               {self writeElems(Args)}
               {self write(vs:@prefix)}
               {self endTag('OMA')}
            end
	 end
	 meth writeOMBVAR(VarList)
	    {self pushTab}
	    {self startTag('OMBVAR')}
            {self writeElems(VarList)}
            {self endTag('OMBVAR' indent:true)}
	    {self popTab}
	 end
	 meth writeOMBIND(X)
            case X of 'OMBIND'(El1 VarList El2) then
               {self startTag('OMBIND')}
               {self writePrefix1} {self writeElem(El1)}
               {self writePrefix1} {self writeOMBVAR(VarList)}
               {self writePrefix1} {self writeElem(El2)}
               {self writePrefix}
               {self endTag('OMBIND')}
            [] 'OMBIND'(El1#'OMBVAR'(VarList)#El2) then % hack for demo
               {self startTag('OMBIND')}
               {self writePrefix1} {self writeElem(El1)}
               {self writePrefix1} {self writeOMBVAR(VarList)}
               {self writePrefix1} {self writeElem(El2)}
               {self writePrefix}
               {self endTag('OMBIND')}
            end
         end
	 meth writeOME(X)
	    'OME'(Sym...) = X
	    Elems = {Value.condSelect X 2 nil} 
	 in
	    {self startTag('OME')}
	    {self write(vs:@prefix#@tab)} {self writeOMS(Sym)}
            {self writeElems(Elems)}
	    {self write(vs:@prefix)}
            {self endTag('OME')}
	 end
         %%
         meth writeOMATP(L)
	    {self pushTab}
	    {self startTag('OMATP')}
            {ForAll L
             proc {$ Sym#Elem}
                {self write(vs:@prefix#@tab)} {self writeOMS(Sym)}
                {self write(vs:@prefix#@tab)} {self writeElem(Elem)}
             end}
	    {self write(vs:@prefix)}
            {self endTag('OMATP')}
	    {self popTab}
	 end
	 meth writeOMATTR(X)
	    case X of 'OMATTR'(L Elem) then
               {self startTag('OMATTR')}
               {self write(vs:@prefix#@tab)} {self writeOMATP(L)}
               {self write(vs:@prefix#@tab)} {self writeElem(Elem)}
               {self write(vs:@prefix)}
               {self endTag('OMATTR')}
            [] 'OMATTR'('OMATP'(L)#Elem) then % hack for demo
               {self startTag('OMATTR')}
               {self write(vs:@prefix#@tab)} {self writeOMATP(L)}
               {self write(vs:@prefix#@tab)} {self writeElem(Elem)}
               {self write(vs:@prefix)}
               {self endTag('OMATTR')}
            end
         end
	 meth writeElem(X)
	    {self pushTab}
	    case {Label X}
	    of 'OMS'    then {self writeOMS(X)}
	    [] 'OMV'    then {self writeOMV(X)}
	    [] 'OMI'    then {self writeOMI(X)}
	    [] 'OMB'    then {self writeOMB(X)}
	    [] 'OMSTR'  then {self writeOMSTR(X)}
	    [] 'OMF'    then {self writeOMF(X)}
	    [] 'OMA'    then {self writeOMA(X)}
	    [] 'OMBIND' then {self writeOMBIND(X)}
	    [] 'OME'    then {self writeOME(X)}
	    [] 'OMATTR' then {self writeOMATTR(X)}
	    end
	    {self popTab}
	 end
	 meth writeOMOBJ(X)
	    'OMOBJ'(Elem) = X in
            {self startTag('OMOBJ')}
	    {self write(vs:@prefix#@tab)} {self writeElem(Elem)}
	    {self write(vs:@prefix)}
            {self endTag('OMOBJ')}
	 end
      end
   end
   
   local   
      fun {WriteElems L WriteElem}
	 case L of nil then ''
	 [] X|R then
	    {WriteElem X}#{WriteElems R WriteElem}
	 end
      end
   in
      class OMWriter

	 attr
	    prefix: ''
	    tab:    '  '
	    
	 meth init(prefix: Prefix <= "\n"
		   tab:    Tab    <= "  ")
	    prefix <- Prefix
	    tab    <- Tab
	 end
	 meth setPrefix(Prefix)
	    prefix <- Prefix
	 end
	 meth setTab(N)
	    Tab = {List.make N}
	    {ForAll Tab proc {$ C} C=&  end}
	 in
	    tab <- Tab
	 end
	 meth shift(Tab<=@tab)
	    prefix <- @prefix#Tab
	 end
	 meth endshift
	    Prefix#_ = @prefix in
	    prefix <- Prefix
	 end
	 %% simple elements
	 meth WriteOMS(X $)
	    'OMS'(cd:CD name:Name) = X in
	    '<OMS cd="'#CD#'" name="'#Name#'"/>'
	 end
	 meth WriteOMV(X $)
	    'OMV'(name:Name) = X in
	    '<OMV name="'#Name#'"/>'
	 end
	 meth WriteOMI(X $)
	    'OMI'(I) = X in
	    '<OMI>'#I#'</OMI>' 
	 end
	 meth WriteOMB(X $)
	    'OMB'(BS) = X in
	    '<OMB>'#BS#'</OMB>'
	 end
	 meth WriteOMSTR(X $)
	    'OMSTR'(BS) = X in
	    '<OMSTR>'#BS#'</OMSTR>'
	 end
	 meth WriteOMF(X $)
	    'OMF'(F) = X in
	    '<OMF dec="'#{Float.toString F}#'"/>'
	 end
	 %% complex elements
	 meth WriteOMA(X $)
	    'OMA'(First Rest) = X in
	    '<OMA>'
	    #@prefix#@tab# OMWriter,WriteElem(First $)
	    #{WriteElems Rest
	     fun {$ Elem} @prefix#@tab# OMWriter,WriteElem(Elem $) end}
	    #@prefix#'</OMA>'
	 end
	 meth WriteOMBVAR(VarList ?VS)
	    OMWriter, shift
	    VS =
	    '<OMBVAR>'
	    #{WriteElems VarList
	     fun {$ Elem} @prefix#@tab# OMWriter,WriteElem(Elem $) end}
	    #@prefix#'</OMBVAR>'
	    OMWriter, endshift
	 end
	 meth WriteOMBIND(X $)
	    'OMBIND'(El1 VarList El2) = X in
	    '<OMBIND>'
	    #@prefix#@tab# OMWriter,WriteElem(El1 $)
	    #@prefix#@tab# OMWriter,WriteOMBVAR(VarList $)
	    #@prefix#@tab# OMWriter,WriteElem(El2 $)
	    #@prefix#'</OMBIND>'
	 end
	 meth WriteOME(X $)
	    'OME'(Sym...) = X
	    Elems = {Value.condSelect X 2 nil} 
	 in
	    '<OME>'
	    #@prefix#@tab# OMWriter,WriteOMS(Sym $)
	    #{WriteElems Elems
	      fun {$ Elem} @prefix#@tab# OMWriter,WriteElem(Elem $) end}
	    #@prefix#'</OME>'
	 end
	 meth WriteOMATP(L ?VS)
	    OMWriter, shift
	    VS =
	    '<OMATP>'
	    #{WriteElems L
	      fun {$ Sym#Elem}
		 @prefix#@tab# OMWriter,WriteOMS(Sym $)
		 #@prefix#@tab# OMWriter,WriteElem(Elem $)
	      end}
	    #@prefix#'</OMATP>'
	    OMWriter, endshift
	 end
	 meth WriteOMATTR(X $)
	    'OMATTR'(L Elem) = X in
	    '<OMATTR>'
	    #@prefix#@tab# OMWriter,WriteOMATP(L $)
	    #@prefix#@tab# OMWriter,WriteElem(Elem $)
	    #@prefix#'</OMATTR>'
	 end
	 meth WriteElem(X ?VS)
	    OMWriter, shift
	    case X
	    of 'OMS'(...)    then OMWriter,WriteOMS(X ?VS) 
	    [] 'OMV'(...)    then OMWriter,WriteOMV(X ?VS)
	    [] 'OMI'(...)    then OMWriter,WriteOMI(X ?VS)
	    [] 'OMB'(...)    then OMWriter,WriteOMB(X ?VS)
	    [] 'OMSTR'(...)  then OMWriter,WriteOMSTR(X ?VS)
	    [] 'OMF'(...)    then OMWriter,WriteOMF(X ?VS)
	    [] 'OMA'(...)    then OMWriter,WriteOMA(X ?VS)
	    [] 'OMBIND'(...) then OMWriter,WriteOMBIND(X ?VS)
	    [] 'OME'(...)    then OMWriter,WriteOME(X ?VS)
	    [] 'OMATTR'(...) then OMWriter,WriteOMATTR(X ?VS)
	    end
	    OMWriter, endshift
	 end
	 meth writeOMOBJ(X $)
	    'OMOBJ'(Elem) = X in
	    '<OMOBJ>'
	    #@prefix#@tab# OMWriter,WriteElem(Elem $)
	    #@prefix#'</OMOBJ>'
	 end
      end
   end
   

   %%
   %% standard format --> internal representation
   %%
   local
      fun {`OMS` X}
	 'OMS'(cd:CD name:Name) = X in
	 sym(cd:CD name:Name)
      end
      fun {`OMV` X}
	 'OMV'(name:Name) = X in
	 var(name:Name)
      end
      fun {`OMI` X}
	 'OMI'(I) = X in
	 int(I)
      end
      fun {`OMB` X}
	 'OMB'(BS) = X in
	 byteString(BS)
      end
      fun {`OMSTR` X}
	 'OMSTR'(BS) = X in
	 str(BS) 
      end
      fun {`OMF` X}
	 'OMF'(F) = X in
	 float(F)
      end
      fun {`OMA` X}
	 'OMA'(First Rest) = X in
	 app({ReadElem First} {Map Rest ReadElem})
      end
      fun {`OMBIND` X}
	 'OMBIND'(El1 VarList El2) = X in
	 bind({ReadElem El1} {Map VarList ReadElem} {ReadElem El2})
      end
      fun {`OME` X}
	 'OME'(Sym...) = X
	 Elements = {Value.condSelect X 2 nil}
      in
	 err({ReadElem Sym} {Map Elements ReadElem}) 
      end
      fun {`OMATTR` X}
	 'OMATTR'(L Elem) = X in
	 {AdjoinAt {ReadElem Elem} attribs
	  {Map L fun {$ Sym#Elem} {ReadElem Sym}#{ReadElem Elem} end}}
      end
      DoRead = unit('OMS':    `OMS`
		    'OMV':    `OMV`
		    'OMI':    `OMI` 
		    'OMB':    `OMB` 
		    'OMSTR':  `OMSTR` 
		    'OMF':    `OMF` 
		    'OMA':    `OMA` 
		    'OMBIND': `OMBIND` 
		    'OME':    `OME` 
		    'OMATTR': `OMATTR`)
      
      fun {ReadElem X}
	 {DoRead.{Label X} X}
      end
   in
      fun {ReadObject X}
	 'OMOBJ'(Elem) = X in
	 object({ReadElem Elem})
      end
   end

   %%
   %% internal representation --> standard format
   %%
   local
      fun {WriteElemSimple X}
	 case X
	 of sym(cd:CD name:Name) then 'OMS'(cd:CD name:Name)
	 [] var(name:Name)       then 'OMV'(name:Name)
	 [] int(I)               then 'OMI'(I)
	 [] byteString(BS)       then 'OMB'(BS)
	 [] str(BS)              then 'OMSTR'(BS)
	 [] float(F)             then 'OMF'(F)
	 [] app(First Rest) then
	    'OMA'({WriteElem First} {Map Rest WriteElem})
	 [] bind(X1 Vars X2) then
	    'OMBIND'({WriteElem X1} {Map Vars WriteElem} {WriteElem X2})
	 [] err(Sym Elements) then
	    'OME'({WriteElem Sym} {Map Elements WriteElem})
	 end
      end
      fun {WriteElem X}
	 if {HasFeature X attribs} then
	    L = {Map X.attribs fun {$ S#E} {WriteElem S}#{WriteElem E} end} in
	    'OMATTR'(L {WriteElemSimple {Record.subtract X attribs}})
	 else
	    {WriteElemSimple X}
	 end
      end
      Writer = {New OMWriter init}
   in
      fun {WriteObject X}
	 object(Y) = X
	 Y1 = {WriteElem Y}
      in
	 {Writer writeOMOBJ('OMOBJ'(Y1) $)}
      end
   end
end



