%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OMDoc2Post.oz ; This file is part of the MathWeb system
%
%   major updates: 23.4.2002
%
%   Authors: Juergen Zimmer
%
%   email: jzimmer@ags.uni-sb.de
%   
% For information about this program, write to:                          
%   MathWeb-Project                                                        
%   AG Siekmann/FB nformatik                                            
%   Universitaet des Saarlandes                                          
%   Bau 36, 4. Stock                                                     
%   D-66041 Saarbruecken                                                 
%   Germany    
%
% For information about the newest version of MathWeb, see 
%   http://www.mathweb.org/mathweb
%
% See the file "LICENSE" or
%    http://www.mathweb.org/mathweb/LICENSE.html
% for information on usage and redistribution
% of this file, and for a DISCLAIMER OF ALL
% WARRANTIES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% This is a one-to-one translation of the XSL style sheet
%%  omdoc/xsl/omdoc2post.xsl. available in the MathWeb sources
%%  or at http://www.mathweb.org/omdoc/xsl/omdoc2post.xsl
%%
%% <xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
%%   xmlns:om="http://www.openmath.org/OpenMath"
%%   xmlns:dc="http://purl.org/DC"
%%   xmlns:omdoc="http://www.mathweb.org/omdoc"
%% version="1.0">

functor

import
   Error
   System
   Property
   Browser(browse: Browse)
   
%   Internet     at 'x-ozlib://mathweb/share/Internet.ozf'

   %% not used anymore:
   Openmath(om2Post: OM2Post)
   at 'x-ozlib://mathweb/omutils/OM2Post.ozf'
   
export
   omdoc2Post: OMDoc2Post
   
define

   SymbolRecord = symbols('arith1': symbols('abs': "absval"
                                            'unary_minus': "change-sign"
                                           )
                          'relation1': symbols('eq': "="
                                               'lt': "less"
                                               'gt': "greater"
                                              )
                         )
   
   {Property.put 'print.depth' 100}
   {Property.put 'print.width' 100}
   
   VS2A = VirtualString.toAtom
   VS2S = VirtualString.toString
   VS2BS = ByteString.make
   
   InfixOpsRec = infix('plus':"+"
                       'minus':"-"
                       'times':"*"
                       'power':"^"
                      )
   InfixOps = {Record.toDictionary InfixOpsRec}
   
   fun {ExceptionToVS ERROR}
      {Error.messageToVirtualString {Error.exceptionToMessage ERROR}}
   end
   
   fun{List2Post Xs Seperator Theory}
      case Xs
      of nil then "NIL"
      [] _|_ then
	 {RecList2Post Xs Seperator Theory}
      end
   end
   
   fun{RecList2Post Term Seperator Theory}
      case Term
      of nil then nil
      [] T|Tr then
         {OMDoc2Post T Theory}
	 #if Tr == nil then nil
	  else 
	     Seperator
	     #{RecList2Post Tr Seperator Theory}
	  end
      else Term end
   end
   
   TYPE      = 'OMS'(cd: {VS2BS "mltt"}
                     name: {VS2BS "type"})
   
   LAMBDA    = 'OMS'(cd: {VS2BS "fns1"}
                     name: {VS2BS "lambda"})
   
   ALL_TYPES = 'OMS'(cd: {VS2BS "mltt"}
                     name: {VS2BS "all-types"})
   
   fun {FindType AList}
      case AList of nil then false
      [] (Key#Val)|Rest then
         case Key
         of !TYPE then Val
         else {FindType Rest}
         end
      end
   end

   fun {WriteType Type}
      "todo"
   end
   
   fun {Var2Post OMV}
      case OMV
      of 'OMATTR'(AList Var) then
         case {FindType AList}
         of false then
            Var.name
         [] Type then
            "("#Var.name#" "#{WriteType Type}#")"
         end
      else
         OMV.name
      end
   end   
   
   fun {Lambda2Post OMBVARs Term}
      case OMBVARs of nil then Term
      [] OMV|Rest then
         "(lam "#{Var2Post OMV}#" "#{Lambda2Post Rest Term}#")"
      end
   end

%   <!-- #################### Theory Elements ##################### -->
   fun{Theory2Post TheoryElement}
      {Browse theory2Post(TheoryElement)}
      case TheoryElement
      of theory(id:          TheoryId
                metadata:    MetaData
                commonnames: CommonNames
                items:       Items
                ...)
         
      then
         Imports = {CondSelect TheoryElement imports nil}
      in
         "\n"#"(th~deftheory "#TheoryId#"\n"
         #if Imports \= nil then
             "(uses "#Imports#")\n"
          else nil end
         #case MetaData.items of Meta|_  then
             case {Filter {Record.toList Meta} fun{$ E} case E of title(...) then true else false end end}
             of nil then nil
             [] title(Title ...) then
                '(help "'#Title
                #(if CommonNames \= nil then
                     CN|_ = CommonNames in
                     case CN
                     of commonname(Name) then
                        " / "#CN
                     else nil end else nil end)
                #'")'
             else nil end
          else nil end
         #")\n"
         #{OMDoc2Post Items TheoryElement}
      else "Unknown theory element" end
   end
   
   fun{PrintCMPs CMPs}
      case CMPs of
         nil then nil
      [] C|Cr then
         {PrintCMPs C}#" "#{PrintCMPs Cr}
      [] 'CMP'(content: Content ...) then
         case {CondSelect CMPs 'xml:lang' unit}
         of unit then '""'
         else {OMDoc2Post Content nil}
         end
      end
   end
   
   %% ================ Axioms =====================
   fun{Axiom2Post AxiomElement Theory}
      case AxiomElement
      of axiom(id: Id
               fmp: FMP
               ...)
      then
         "(th~defaxiom "#Id#"\n"
         #"  (in "#Theory.id#")\n"
         #nil  %% ToDo: (termdecl) things!
         #" (formula "#{OM2Post FMP}#")\n"
         #case  {CondSelect AxiomElement cmps nil}
          of nil then nil
          [] CMPs then " (help "#{PrintCMPs CMPs}#")\n" end
         #")\n"
      else nil end
   end
   
   %% ================ Assertions =====================
   fun{Assertion2Post AssertionElement Theory}
      case AssertionElement
      of assertion(id: Id
                   fmp: FMP
                   type: Type
                   ...) then
         RealTheory = {CondSelect AssertionElement theory Theory}
         Symbols = {CondSelect  AssertionElement symbols nil}
         TypeConstants = {Filter Symbols fun{$ S} S.kind = "type" end}
         Constants = {Filter Symbols fun{$ S} S.kind = "object" end}
      in
         case Type
         of "theorem" then "(th~deftheorem "
         else              "(th~defproblem " end
         #Id#"\n"
         #" (in "#RealTheory#")\n"
         #nil     %% ToDO (termdecl)
         %% note: we don't use mode="inner" because MP said we don't need it.
         #if TypeConstants \= nil then
             "    (type-constants "#{OMDoc2Post TypeConstants Theory}#")\n"
          end
         #if Constants \= nil then
             "    (type-constants "#{OMDoc2Post Constants Theory}#")\n"
          end
         #{OMDoc2Post {CondSelect AssertionElement fmp nil} Theory}
         #case  {CondSelect AssertionElement cmps nil}
          of nil then nil
          [] CMPs then " (help "#{PrintCMPs CMPs}#")\n" end
         #")\n"
      else nil end
   end
   

   %% <!-- ################ Symbols ################ -->

   fun{Symbol2Post SymbolElement Theory}
%      {Browse symbol2Post(SymbolElement)}
      case SymbolElement
      of symbol(id: SymbolId
                scope: Scope
                kind: Kind
                types: Types
                ...) then
         %% local scope
         case {VS2S Scope} of "local" then
            case Kind of "type" then
               {CondSelect SymbolElement commonname nil}
               #" "
            [] "object" then
               "("#SymbolId#" "#{OMDoc2Post {CondSelect SymbolElement type nil} Theory}#")"
            else {Browse 'not implemented local symbol '#SymbolElement}
               nil
            end
            
            %% global scope
         [] "global" then
            {Browse globalSymbol}
            case {VS2S Kind}
            of "type" then
               {Browse kind(type)}
               {Browse inTheory(Theory.id)}
               Types = {Filter {CondSelect SymbolElement types nil} fun{$ Type}
                                                                       case {CondSelect Type system unit}
                                                                       of unit then false
                                                                       [] S then {VS2S S} == "POST"
                                                                       end
                                                                    end}
            in
               {Browse types(Types)}
               "(th~deftype "
               #SymbolId#"\n"
               #"  (in "#Theory.id#")\n"
%               #if Types \= nil then
               %% TODO.... we can't handle the arguments number now.
               #" (arguments 0))"
            else
               DefinitionsFor = {Filter Theory.items
                                 fun{$ I}
                                    case I
                                    of definition('for': !SymbolId ...) then
                                       true else false
                                    end
                                 end}
            in
               case DefinitionsFor
               of definition(fmp: FMP cmps: CMPs ...)|_ then
                  "(th~defdef "#SymbolId#"\n"
                  #"  (in "#Theory.id#")\n"
                  % TODO: we don't handle privates now...
%                   #local Privates = {Filter Theory.items
%                                      fun{$ I}
%                                         case I
%                                         of private('for': !SymbolId ...) then
%                                            true else false
%                                         end
%                                      end}
%                    in
%                       case Privates of nil then nil
%                       [] P|_ then
                  #"\n  (definition "
                  #{OM2Post FMP}
                  #")\n"
                  #"  (help "
                  #case CMPs 
                   of cmp(content: Content ...)|_ then Content else '""' end
                  #"))\n"
               else 
                  case Types of nil then "Can't handle constant without types"
                  [] Type|_ then 
                     "(th~defconstant "
                     #SymbolId#"\n   (in "#Theory.id#")"
%                   #local Privates = {Filter Theory.items
%                                      fun{$ I}
%                                         case I
%                                         of private('for': !SymbolId ...) then
%                                            true else false
%                                         end
%                                      end}
%                    in
%                       case Privates of nil then nil
%                       [] P|_ then
                  % TODO: we don't handle privates here...
                     #"  (type "#{OMDoc2Post Type Theory}#")\n"
                     #")\n"
                  else
                     "Pathological case with types for symbol "#SymbolId
                  end
               end
            end
         end
      else
         {Browse 'cant handle'(SymbolElement)}
         nil
      end
   end
                 
   fun{OMDoc2Post Element Theory}
%      {Browse omdoc2Post(Element)}
      if {HasFeature Theory id} then {Browse inTheory(Theory)} end
      case Element
      of nil then nil
      [] _|_ then
         {List2Post Element "\n" Theory}
      [] omdoc(items: Items ...) then
         {OMDoc2Post Items Theory}
      [] theory(...) then
         {Theory2Post Element}
      [] symbol(...) then
         R = {Symbol2Post Element Theory}
      in {Browse R}
         R 
      [] axiom(...) then
         {Axiom2Post Element Theory}
      [] assumption(id: Id
                    omobj: OMObj ...) then
         " (assumption "#Id#" "#{OM2Post OMObj}#")\n"
      [] conclusion(id: Id
                    omobj: OMObj ...) then 
         " (conclusion "#Id#" "#{OM2Post OMObj}#")\n"
      [] private(type: Type data: Data ...) then
         "  ("#Type#" "#Data#")\n"
      [] metacomment(...) then
         nil  % ToDO: metacomment(....)
      [] hypothesis(id: Id fmp: FMP ...) then
         "    ("#Id#" ("#Id#")"
         #{OMDoc2Post FMP Theory}
         #'(0 ("HYP" () () "grounded" () ())))\n'
      [] derive(...) then %% ToDO: derive
         nil
      [] conclude(...) then %% TODo: conclude
         nil
      [] method(...) then %%ToDO: method
         nil
      [] type(omobj: OMObj ...) then
         {Browse hallo}
         R = {OM2Post OMObj}
      in {Browse R} R 
         
%       [] 'OMA'('OMS'(name: "list" cd: "list1") Args) then
%          {OMDoc2Post Args}
%       [] 'OMS'(name: Name cd: CD) then
%          NameA = {VS2A Name}
%          CDA = {VS2A CD}
%       in
%          if {HasFeature SymbolRecord CDA} andthen {HasFeature SymbolRecord.CDA NameA} then 
%             (SymbolRecord.CDA).NameA
%          else
%             NameA
%          end
%       [] 'OMV'(name: Name) then
%          {VS2A Name}
%       [] 'OMA'(Functor Args) then
%          %%{Browse 'OMA'('functor':Functor args:Args)}
%          "("#{OMDoc2Post Functor}
%          #" "
%          #if {IsList Args}
%           then {RecList2Post Args}
%           else {RecList2Post {Record.toList Args}}
%           end
%          #")"
%       [] 'OMI'(I) then
%          I
%       [] 'OMBIND'(OMS OMBVARs Term) then
%          case OMS of !LAMBDA then
%             {Lambda2Post OMBVARs {OMDoc2Post Term}}
%          [] !ALL_TYPES then
%             %% skip all-types/kappa for now
%             {OMDoc2Post Term}
%          else
%             raise OMObj end
%          end
%       elseif {IsVirtualString OMObj} then
%          {VS2A OMObj}
      else
         raise Element end
      end
   end
   
          
end %define
   



