%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OMDoc2Tptp.oz ; This file is part of the MathWeb system
%
%   major updates: 23.4.2002
%
%   Authors: Juergen Zimmer
%
%   email: jzimmer@ags.uni-sb.de
%   
% For information about this program, write to:                          
%   MathWeb-Project                                                        
%   AG Siekmann/FB nformatik                                            
%   Universitaet des Saarlandes                                          
%   Bau 36, 4. Stock                                                     
%   D-66041 Saarbruecken                                                 
%   Germany    
%
% For information about the newest version of MathWeb, see 
%   http://www.mathweb.org/mathweb
%
% See the file "LICENSE" or
%    http://www.mathweb.org/mathweb/LICENSE.html
% for information on usage and redistribution
% of this file, and for a DISCLAIMER OF ALL
% WARRANTIES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%

functor

import
   Error
   System
   Property
   Browser(browse: Browse)
   
%   Internet     at 'x-ozlib://mathweb/share/Internet.ozf'

   %% not used anymore:
   Openmath(om2Post: OM2Post)
   at 'x-ozlib://mathweb/omutils/OM2Post.ozf'
   
export
   omdoc2tptp: OMDoc2Tptp
   
define

   local
      fun {DoEnumerate Xs Sep F}
         case Xs of nil then ''
         [] X|Xr then
            Sep#{F X}#{DoEnumerate Xr Sep F}
         end
      end
   in
      fun {Enumerate Xs Sep F}
         case Xs of nil then
            {Exception.raiseError enumerate(emptyList)} unit
         [] X|Xr then
            {F X}#{DoEnumerate Xr Sep F}
         end
      end
   end

   SymbolRecord = symbols('arith1': symbols('abs': "absval"
                                            'unary_minus': "change-sign"
                                           )
                          'relation1': symbols('eq': "="
                                               'lt': "less"
                                               'gt': "greater"
                                              )
                         )
   
   {Property.put 'print.depth' 100}
   {Property.put 'print.width' 100}
   
   VS2A = VirtualString.toAtom
   VS2S = VirtualString.toString
   VS2BS = ByteString.make
   
   InfixOpsRec = infix('plus':"+"
                       'minus':"-"
                       'times':"*"
                       'power':"^"
                      )
   InfixOps = {Record.toDictionary InfixOpsRec}
   
   fun {ExceptionToVS ERROR}
      {Error.messageToVirtualString {Error.exceptionToMessage ERROR}}
   end
   
   fun{List2Tptp Xs Seperator}
      case Xs
      of nil then nil
      [] _|_ then
	 {RecList2Tptp Xs Seperator }
      end
   end
   
   fun{RecList2Tptp Term Seperator}
      case Term
      of nil then nil
      [] T|Tr then
         {OM2Tptp T}
	 #if Tr == nil then nil
	  else 
	     Seperator
	     #{RecList2Tptp Tr Seperator}
	  end
      else Term end
   end
   
   TYPE      = 'OMS'(cd: {VS2BS "mltt"}
                     name: {VS2BS "type"})
   
   LAMBDA    = 'OMS'(cd: {VS2BS "fns1"}
                     name: {VS2BS "lambda"})
   
   ALL_TYPES = 'OMS'(cd: {VS2BS "mltt"}
                     name: {VS2BS "all-types"})

   fun {Sym CD#Name}
      'OMS'(cd: {VS2BS CD} name: {VS2BS Name})
   end
   
   FORALL    = {Sym "quant1"#"forall"}
   EXISTS    = {Sym "quant1"#"exists"}   
   IMPLIES   = {Sym "logic1"#"implies"}
   EQUIV     = {Sym "logic1"#"equivalent"}
   AND       = {Sym "logic1"#"and"}
   OR        = {Sym "logic1"#"or"}
   NOT       = {Sym "logic1"#"not"}
   TRUE      = {Sym "logic1"#"true"}
   FALSE     = {Sym "logic1"#"false"}
   EQUALITY  = {Sym "relation1"#"eq"}
   
   fun {FindType AList}
      case AList of nil then false
      [] (Key#Val)|Rest then
         case Key
         of !TYPE then Val
         else {FindType Rest}
         end
      end
   end

   fun {WriteType Type}
      "todo"
   end
   
   fun {Var2Tptp OMV}
      case OMV
      of 'OMATTR'(AList Var) then
         case {FindType AList}
         of false then
            Var.name
         [] Type then
            "("#Var.name#" "#{WriteType Type}#")"
         end
      else
         OMV.name
      end
   end   
   
   fun {Lambda2Tptp OMBVARs Term}
      case OMBVARs of nil then Term
      [] OMV|Rest then
         "(lam "#{Var2Tptp OMV}#" "#{Lambda2Tptp Rest Term}#")"
      end
   end

   fun{OMObj2Tptp OMObj Name Type}
      "input_formula("#Name#", "#Type#" , ("
      #"\n"
      #{OM2Tptp OMObj}
      #")).\n\n"
   end
         
   fun{OM2Tptp OMObj}
%      {Browse oM2LClam(OMObj)}
      case OMObj
      of nil then "nil"
      [] _|_ then
         {List2Tptp OMObj "\n"}
      [] 'OMOBJ'(Obj) then
         {OM2Tptp Obj}
      [] 'OMA'('OMS'(name: "list" cd: "list1") Args) then
         {OM2Tptp Args}
      [] 'OMS'(name: Name cd: CD) then
         NameA = {VS2A Name}
      in
         NameA 
      [] 'OMV'(name: Name) then
         {VS2A Name}
      [] 'OMA'(!IMPLIES [LHS RHS]) then
         "("#{OM2Tptp LHS}
         #"\n"
         #" => "#{OM2Tptp RHS}
         #")"
      [] 'OMA'(!AND [LHS RHS]) then
         "("#{OM2Tptp LHS}
         #"\n"
         #" & "#{OM2Tptp RHS}
         #")"
      [] 'OMA'(!EQUIV [LHS RHS]) then
         "("#{OM2Tptp LHS}
         #"\n"
         #" <=> "#{OM2Tptp RHS}
         #")"
      [] 'OMA'(!NOT [FORMULA]) then
         " ~ ("#{OM2Tptp FORMULA}
         #")"
      [] 'OMA'(!EQUALITY [LHS RHS]) then
         "equal("#{OM2Tptp LHS}
         #", "#{OM2Tptp RHS}
         #")"
      [] 'OMA'(Functor Args...) then
	 Func = {OM2Tptp Functor}
      in
         Func#"("
         #{RecList2Tptp Args ", "}
         #")"
      [] 'OMI'(I) then
         I
      [] 'OMBIND'(OMS OMBVARs Term) then
         case OMS
         of !FORALL then
            "! ["
            #{RecList2Tptp OMBVARs ", "}
            #"] : \n"
            #{OM2Tptp Term}
         [] !EXISTS then
            "? ["
            #{RecList2Tptp OMBVARs ", "}
            #"] : \n"
            #{OM2Tptp Term}
         else
            OMObj 
         end
      elseif {IsVirtualString OMObj} then
         {VS2A OMObj}
      else
         OMObj
      end
   end

   local
      fun {SpecialCharToUnderscore C}
         if {Char.isAlNum C} then C else &_ end
      end
   in
      fun {SpecialToUnderscore VS}
         {Map {VS2S VS} SpecialCharToUnderscore}
      end
   end
   
%   <!-- #################### Theory Elements ##################### -->
   fun{Theory2Tptp TheoryElement}
%      {Browse theory2Tptp(TheoryElement)}
      case TheoryElement
      of theory( %id:          TheoryId
                %metadata:    MetaData
                %commonnames: CommonNames
                items:       Items
                ...)
         
      then
         fun {FilterLabel Xs L}
            {Filter Xs fun {$ X} {Label X}==L end}
         end
         Axioms = {FilterLabel Items axiom}

         Assertion = case {FilterLabel Items assertion}
                     of nil then unit
                     [] A|_ then A end
         AxiomFormulas = {Map Axioms fun{$ Axiom}
                                        {SpecialToUnderscore Axiom.id}
                                        #Axiom.fmp.omobj
                                     end}
         AssertionFormula = {SpecialToUnderscore Assertion.id}#Assertion.fmp.omobj
         
      in
         {Enumerate AxiomFormulas "\n"
          fun {$ ID#OMOBJ}
             {OMObj2Tptp OMOBJ ID axiom}
          end}          
         #{OMObj2Tptp AssertionFormula.2 AssertionFormula.1 conjecture}
      else "Unknown theory element" end
   end
   

   fun{OMDoc2Tptp Element Theory}
%      {Browse omdoc2Tptp(Element)}
      if {HasFeature Theory id} then {Browse inTheory(Theory)} end
      case Element
      of nil then nil
      [] I|_ then
         {OMDoc2Tptp I Theory}
      [] omdoc(items: Items ...) then
         {OMDoc2Tptp Items Theory}
      [] theory(...) then
         {Theory2Tptp Element}
      [] symbol(...) then
         R = {Symbol2Tptp Element Theory}
      in
         %{Browse R}
         R 
%       [] axiom(...) then
%          {Axiom2Tptp Element Theory}
%       [] assumption(id: Id
%                     omobj: OMObj ...) then
%          " (assumption "#Id#" "#{OM2Post OMObj}#")\n"
%       [] conclusion(id: Id
%                     omobj: OMObj ...) then 
%          " (conclusion "#Id#" "#{OM2Post OMObj}#")\n"
%       [] private(type: Type data: Data ...) then
%          "  ("#Type#" "#Data#")\n"
%       [] metacomment(...) then
%          nil  % ToDO: metacomment(....)
%       [] hypothesis(id: Id fmp: FMP ...) then
%          "    ("#Id#" ("#Id#")"
%          #{OMDoc2Tptp FMP Theory}
%          #'(0 ("HYP" () () "grounded" () ())))\n'
%       [] derive(...) then %% ToDO: derive
%          nil
%       [] conclude(...) then %% TODo: conclude
%          nil
%       [] method(...) then %%ToDO: method
%          nil
%       [] type(omobj: OMObj ...) then
%          {Browse hallo}
%          R = {OM2Post OMObj}
%       in {Browse R} R 
         
% %       [] 'OMA'('OMS'(name: "list" cd: "list1") Args) then
% %          {OMDoc2Tptp Args}
% %       [] 'OMS'(name: Name cd: CD) then
% %          NameA = {VS2A Name}
% %          CDA = {VS2A CD}
% %       in
% %          if {HasFeature SymbolRecord CDA} andthen {HasFeature SymbolRecord.CDA NameA} then 
% %             (SymbolRecord.CDA).NameA
% %          else
% %             NameA
% %          end
% %       [] 'OMV'(name: Name) then
% %          {VS2A Name}
% %       [] 'OMA'(Functor Args) then
% %          %%{Browse 'OMA'('functor':Functor args:Args)}
% %          "("#{OMDoc2Tptp Functor}
% %          #" "
% %          #if {IsList Args}
% %           then {RecList2Tptp Args}
% %           else {RecList2Tptp {Record.toList Args}}
% %           end
% %          #")"
% %       [] 'OMI'(I) then
% %          I
% %       [] 'OMBIND'(OMS OMBVARs Term) then
% %          case OMS of !LAMBDA then
% %             {Lambda2Tptp OMBVARs {OMDoc2Tptp Term}}
% %          [] !ALL_TYPES then
% %             %% skip all-types/kappa for now
% %             {OMDoc2Tptp Term}
% %          else
% %             raise OMObj end
% %          end
% %       elseif {IsVirtualString OMObj} then
% %          {VS2A OMObj}
      else
         raise Element end
      end
   end
   %% <!-- ################ Symbols ################ -->

   fun{Symbol2Tptp SymbolElement Theory}
%      {Browse symbol2Tptp(SymbolElement)}
      case SymbolElement
      of symbol(id: SymbolId
                scope: Scope
                kind: Kind
                types: Types
                ...) then
         %% local scope
         case {VS2S Scope} of "local" then
            case Kind of "type" then
               {CondSelect SymbolElement commonname nil}
               #" "
            [] "object" then
               "("#SymbolId#" "#{OMDoc2Tptp {CondSelect SymbolElement type nil} Theory}#")"
            else {Browse 'not implemented local symbol '#SymbolElement}
               nil
            end
            
            %% global scope
         [] "global" then
            {Browse globalSymbol}
            case {VS2S Kind}
            of "type" then
               {Browse kind(type)}
               {Browse inTheory(Theory.id)}
               Types = {Filter {CondSelect SymbolElement types nil} fun{$ Type}
                                                                       case {CondSelect Type system unit}
                                                                       of unit then false
                                                                       [] S then {VS2S S} == "POST"
                                                                       end
                                                                    end}
            in
               {Browse types(Types)}
               "(th~deftype "
               #SymbolId#"\n"
               #"  (in "#Theory.id#")\n"
%               #if Types \= nil then
               %% TODO.... we can't handle the arguments number now.
               #" (arguments 0))"
            else
               DefinitionsFor = {Filter Theory.items
                                 fun{$ I}
                                    case I
                                    of definition('for': !SymbolId ...) then
                                       true else false
                                    end
                                 end}
            in
               case DefinitionsFor
               of definition(fmp: FMP cmps: CMPs ...)|_ then
                  "(th~defdef "#SymbolId#"\n"
                  #"  (in "#Theory.id#")\n"
                  % TODO: we don't handle privates now...
%                   #local Privates = {Filter Theory.items
%                                      fun{$ I}
%                                         case I
%                                         of private('for': !SymbolId ...) then
%                                            true else false
%                                         end
%                                      end}
%                    in
%                       case Privates of nil then nil
%                       [] P|_ then
                  #"\n  (definition "
                  #{OM2Post FMP}
                  #")\n"
                  #"  (help "
                  #case CMPs 
                   of cmp(content: Content ...)|_ then Content else '""' end
                  #"))\n"
               else 
                  case Types of nil then "Can't handle constant without types"
                  [] Type|_ then 
                     "(th~defconstant "
                     #SymbolId#"\n   (in "#Theory.id#")"
%                   #local Privates = {Filter Theory.items
%                                      fun{$ I}
%                                         case I
%                                         of private('for': !SymbolId ...) then
%                                            true else false
%                                         end
%                                      end}
%                    in
%                       case Privates of nil then nil
%                       [] P|_ then
                  % TODO: we don't handle privates here...
                     #"  (type "#{OMDoc2Tptp Type Theory}#")\n"
                     #")\n"
                  else
                     "Pathological case with types for symbol "#SymbolId
                  end
               end
            end
         end
      else
         {Browse 'cant handle'(SymbolElement)}
         nil
      end
   end
                 
   
          
end %define
   



