%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OMPrintOz.oz ; This file is part of the MathWeb system
%
%   major updates: 19.4.2002
%
%   Authors: Juergen Zimmer
%
%   email: jzimmer@ags.uni-sb.de
%   
% For information about this program, write to:                          
%   MathWeb-Project                                                        
%   AG Siekmann/FB nformatik                                            
%   Universitaet des Saarlandes                                          
%   Bau 36, 4. Stock                                                     
%   D-66041 Saarbruecken                                                 
%   Germany    
%
% For information about the newest version of MathWeb, see 
%   http://www.mathweb.org/mathweb
%
% See the file "LICENSE" or
%    http://www.mathweb.org/mathweb/LICENSE.html
% for information on usage and redistribution
% of this file, and for a DISCLAIMER OF ALL
% WARRANTIES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% printer for Oz representation of openmath objects 
%%
%%  pre-alpha state... not running!

functor
   
import
   System
   
export
   'class':     NewOMWriter
   
define
   
   %%
   %% write OpenMath objects as  Oz record trees
   %%
   local
      skip
   in
      class NewOMWriter 
         
         feat
            app
	 attr
            indent: false
            prefix: ''
	    tab:    '  '
	    
	 meth init(app:    App    <=  System
                   prefix: Prefix <= "\n"
		   tab:    Tab    <= "  "
                   indent: Indent <= true
                  )
	    prefix <- Prefix
	    tab    <- Tab
            indent <- Indent
            self.app = App
	 end
         %%
         meth indentOn
            indent <- true
         end
         meth indentOff
            indent <- false
         end
         %%
	 meth setPrefix(Prefix)
	    prefix <- Prefix
	 end
         %%
         meth setTab(N)
	    Tab = {List.make N} in
	    {ForAll Tab proc {$ C} C=&  end}
	    tab <- Tab
	 end
         meth pushTab(Tab<=@tab)
	    prefix <- @prefix#Tab
	 end
	 meth popTab
	    Prefix#_ = @prefix in
	    prefix <- Prefix
	 end
         %%
         meth writePrefix
            if @indent then {self.app.showInfo @prefix} end
         end
         meth writePrefix1
            if @indent then {self.app.showInfo @prefix#@tab} end
         end
         %%
         meth startTag(...) = StartTag
            {self writePrefix1}
         end
         meth endTag(...) = EndTag
            {self writePrefix}
         end

         meth tag(T)
            {self writePrefix1}
            {self.app.showInfo "\'"#{Label T}#"\'("}
            {ForAll {Filter {Arity T} IsInt}
             proc {$ I}
                {self.app.showInfo I#": \""#T.I#" "}
             end}
            {self.app.showInfo ")"}
         end

         %%
         meth write(vs: VS)
            {self.app.printInfo VS}
         end
         %%
         meth writeElems(L)
            {ForAll L
             proc {$ Elem}
                {self write(vs:@prefix#@tab)}
                {self writeElem(Elem)}
             end}
         end
         
         %% simple elements
	 meth writeOMS(X)
	    {self tag(X)}  % 'OMS'(cd:CD name:Name)
	 end
	 meth writeOMV(X)
	    {self tag(X)} % 'OMV'(name:Name)
	 end
	 meth writeOMI(X)
	    {self tag(X)} % 'OMI'(I)
	 end
	 meth writeOMB(X)
	    {self tag(X)} % 'OMB'(BS)
	 end
	 meth writeOMSTR(X)
	    {self tag(X)} % 'OMSTR'(BS)
	 end
	 meth writeOMF(X)
	    'OMF'(F) = X in
	    {self tag('OMF'(dec:{Float.toString F}))}
	 end
         %% complex elements
	 meth writeOMA(X)
            case X
            of 'OMA'(F Args) then 
               {self.app.showInfo "\'OMA\'("}
               {self.app.showInfo @prefix#@tab}
               {self writeElem(F)}
               {self writeElems(Args)}
               {self.app.showInfo @prefix}
               {self.app.showInfo ")"}
            else
               skip
            end
         end
	 meth writeOMBVAR(VarList)
% 	    {self pushTab}
% 	    {self startTag('OMBVAR')}
%             {self writeElems(VarList)}
%             {self endTag('OMBVAR' indent:true)}
% 	    {self popTab}
            skip
         end
	 meth writeOMBIND(X)
            case X of 'OMBIND'(El1 VarList El2) then
               {self startTag('OMBIND')}
               {self writePrefix1} {self writeElem(El1)}
               {self writePrefix1} {self writeOMBVAR(VarList)}
               {self writePrefix1} {self writeElem(El2)}
               {self writePrefix}
               {self endTag('OMBIND')}
            [] 'OMBIND'(El1#'OMBVAR'(VarList)#El2) then % hack for demo
               {self startTag('OMBIND')}
               {self writePrefix1} {self writeElem(El1)}
               {self writePrefix1} {self writeOMBVAR(VarList)}
               {self writePrefix1} {self writeElem(El2)}
               {self writePrefix}
               {self endTag('OMBIND')}
            end
         end
	 meth writeOME(X)
	    'OME'(Sym...) = X
	    Elems = {Value.condSelect X 2 nil} 
	 in
	    {self startTag('OME')}
	    {self write(vs:@prefix#@tab)} {self writeOMS(Sym)}
            {self writeElems(Elems)}
	    {self write(vs:@prefix)}
            {self endTag('OME')}
	 end
         %%
         meth writeOMATP(L)
	    {self pushTab}
	    {self startTag('OMATP')}
            {ForAll L
             proc {$ Sym#Elem}
                {self write(vs:@prefix#@tab)} {self writeOMS(Sym)}
                {self write(vs:@prefix#@tab)} {self writeElem(Elem)}
             end}
	    {self write(vs:@prefix)}
            {self endTag('OMATP')}
	    {self popTab}
	 end
	 meth writeOMATTR(X)
	    case X of 'OMATTR'(L Elem) then
               {self startTag('OMATTR')}
               {self write(vs:@prefix#@tab)} {self writeOMATP(L)}
               {self write(vs:@prefix#@tab)} {self writeElem(Elem)}
               {self write(vs:@prefix)}
               {self endTag('OMATTR')}
            [] 'OMATTR'('OMATP'(L)#Elem) then % hack for demo
               {self startTag('OMATTR')}
               {self write(vs:@prefix#@tab)} {self writeOMATP(L)}
               {self write(vs:@prefix#@tab)} {self writeElem(Elem)}
               {self write(vs:@prefix)}
               {self endTag('OMATTR')}
            end
         end
	 meth writeElem(X)
	    {self pushTab}
	    case {Label X}
	    of 'OMS'    then {self writeOMS(X)}
	    [] 'OMV'    then {self writeOMV(X)}
	    [] 'OMI'    then {self writeOMI(X)}
	    [] 'OMB'    then {self writeOMB(X)}
	    [] 'OMSTR'  then {self writeOMSTR(X)}
	    [] 'OMF'    then {self writeOMF(X)}
	    [] 'OMA'    then {self writeOMA(X)}
	    [] 'OMBIND' then {self writeOMBIND(X)}
	    [] 'OME'    then {self writeOME(X)}
	    [] 'OMATTR' then {self writeOMATTR(X)}
	    end
	    {self popTab}
	 end
	 meth writeOMOBJ(X)
	    'OMOBJ'(Elem) = X in
            {self startTag('OMOBJ')}
	    {self write(vs:@prefix#@tab)} {self writeElem(Elem)}
	    {self write(vs:@prefix)}
            {self endTag('OMOBJ')}
	 end
      end
   end
   
end



