%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Omdoc2Tps.oz ; This file is part of the MathWeb system
%
%   major updates: 19.4.2001
%
%   Authors: Juergen Zimmer
%
%   email: jzimmer@ags.uni-sb.de
%   
% For information about this program, write to:                          
%   MathWeb-Project                                                        
%   AG Siekmann/FB nformatik                                            
%   Universitaet des Saarlandes                                          
%   Bau 36, 4. Stock                                                     
%   D-66041 Saarbruecken                                                 
%   Germany    
%
% For information about the newest version of MathWeb, see 
%   http://www.mathweb.org/mathweb
%
% See the file "LICENSE" or
%    http://www.mathweb.org/mathweb/LICENSE.html
% for information on usage and redistribution
% of this file, and for a DISCLAIMER OF ALL
% WARRANTIES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Useful functions for OMDoc handling
%%
%%  STILL IN A BAD STATE!
functor

import
   Error
   System
   Property
   Browser(browse: Browse)
   
%   Internet     at 'x-ozlib://mathweb/share/Internet.ozf'
   
export
   omdoc2Tps:      Omdoc2Tps
   
define
   SymbolRecord = symbols('arith1': symbols('abs': "absval"
                                            'unary_minus': "change-sign"
                                           )
                          'relation1': symbols('eq': "="
                                               'lt': "less"
                                               'gt': "greater"
                                              )
                         )

   InfixOpsRec = infix('OR': "OR"#true
                       'AND':"AND"#true
                       'IMPLIES':"IMPLIES"#true
                       'power':"^"#true
                       'leq': "<="#true
                       'geq': ">="#true
                       'lt': "<"#true
                       'gt': ">"#true
                       'eq': "="#false
		       'funtype': "->"#true
		      )
   InfixOps = {Record.toDictionary InfixOpsRec}
   
   {Property.put 'print.depth' 100}
   {Property.put 'print.width' 100}
   
   VS2A = VirtualString.toAtom
   VS2S = VirtualString.toString
   VS2BS = ByteString.make
   
   InfixOpsRec = infix('plus':"+"
                       'minus':"-"
                       'times':"*"
                       'power':"^"
                      )
   InfixOps = {Record.toDictionary InfixOpsRec}
   
   fun {ExceptionToVS ERROR}
      {Error.messageToVirtualString {Error.exceptionToMessage ERROR}}
   end
   
   fun{List2Tps Xs}
      case Xs
      of nil then "NIL"
      [] _|_ then
	 "("
	 #{RecList2Tps Xs}
	 #")"
      end
   end
   
   fun{RecList2Tps Term}
      case Term
      of nil then nil
      [] T|Tr then
         {OM2Tps T}
	 #if Tr == nil then ""
	  else 
	     " "
	     #{RecList2Tps Tr}
	  end
      else Term end
   end
   
   TYPE      = 'OMS'(cd: {VS2BS "mltt"}
                     name: {VS2BS "type"})
   
   LAMBDA    = 'OMS'(cd: {VS2BS "fns1"}
                     name: {VS2BS "lambda"})
   
   ALL_TYPES = 'OMS'(cd: {VS2BS "mltt"}
                     name: {VS2BS "all-types"})
   
   fun {FindType AList}
      case AList of nil then false
      [] (Key#Val)|Rest then
         case Key
         of !TYPE then Val
         else {FindType Rest}
         end
      end
   end

   fun {WriteType Type}
      "todo"
   end
   
   fun {Var2Tps OMV}
      case OMV
      of 'OMATTR'(AList Var) then
         case {FindType AList}
         of false then
            Var.name
         [] Type then
            "("#Var.name#" "#{WriteType Type}#")"
         end
      else
         OMV.name
      end
   end   
   
   fun {Lambda2Tps OMBVARs Term}
      case OMBVARs of nil then Term
      [] OMV|Rest then
         "(lam "#{Var2Tps OMV}#" "#{Lambda2Tps Rest Term}#")"
      end
   end
   
   fun{OM2Tps OMObj}
%      {Browse om2Tps(OMObj)}
      case OMObj
      of nil then ""
      [] _|_ then
         {List2Tps {Map OMObj OM2Tps}}
      [] 'OMOBJ'(Obj) then
         {OM2Tps Obj}
      [] 'OMA'('OMS'(name: "list" cd: "list1") Args) then
         {OM2Tps Args}
      [] 'OMS'(name: Name cd: CD) then
         NameA = {VS2A Name}
         CDA = {VS2A CD}
      in
         if {HasFeature SymbolRecord CDA} andthen {HasFeature SymbolRecord.CDA NameA} then 
            (SymbolRecord.CDA).NameA
         else
            NameA
         end
      [] 'OMV'(name: Name) then
         {VS2A Name}
      [] 'OMA'(Functor Args...) then
	 Func = {OM2Tps Functor}
      in
	 case if {IsAtom Func} then
		 {Dictionary.condGet InfixOps Func unit}
	      else unit end
         of unit then 
            Func
            #"("
	    #{RecList2Tps Args  FuncDict}
            #")"
         [] Op#Parans then
            case Args
            of [A1 A2] then                
	       if Parans then "(" else "" end
               #{OM2Tps A1}
	       #Op
               #{OM2Tps A2}
         %%{Browse 'OMA'('functor':Functor args:Args)}
         "("#{OM2Tps Functor}
         #" "
         #if {IsList Args}
          then {RecList2Tps Args}
          else {RecList2Tps {Record.toList Args}}
          end
         #")"
      [] 'OMI'(I) then
               I
            [] 'OMBIND'(OMS OMBVARs Term) then
               case OMS of !LAMBDA then
                  {Lambda2Tps OMBVARs {OM2Tps Term}}
               [] !ALL_TYPES then
            %% skip all-types/kappa for now
                  {OM2Tps Term}
               else
                  raise OMObj end
               end
            elseif {IsVirtualString OMObj} then
               {VS2A OMObj}
            else
               raise OMObj end
      end
   end
   
   fun{List2TermPrinter Xs}
      case Xs
      of nil then "NIL"
      [] _|_ then
	 "("
	 #{RecList2TermPrinter Xs}
	 #")"
      end
   end
   
   fun{RecList2TermPrinter Term}
      case Term
      of nil then nil
      [] T|Tr then
	 {OM2TermPrinter T}
	 #if Tr == nil then ""
	  else 
	     " "
	     #{RecList2TermPrinter Tr}
	  end
      else Term end
   end

   fun {Var2TermPrinter OMV}
      case OMV
      of 'OMATTR'(AList Var) then
         case {FindType AList}
         of false then
            Var.name
         [] Type then
            "["#Var.name#" "#{WriteType Type}#"]"
         end
      else
         OMV.name
      end
   end
   
   fun {Lambda2TermPrinter OMBVARs Term}
      case OMBVARs of nil then Term
      [] OMV|Rest then
         "(lam "#{Var2TermPrinter OMV}#" "#{Lambda2TermPrinter Rest Term}#")"
      end
   end
      
   fun{OM2TermPrinter OMObj}
%      {Browse om2TermPrinter(OMObj)}
      case OMObj
      of nil then ""
      [] _|_ then
         {List2TermPrinter {Map OMObj OM2TermPrinter}}
      [] 'OMOBJ'(Obj) then
         {OM2TermPrinter Obj}
      [] 'OMA'('OMS'(name: "list" cd: "list1") Args) then
         {OM2TermPrinter Args}
      [] 'OMS'(name: Name cd: CD) then
         NameA = {VS2A Name}
         CDA = {VS2A CD}
      in
         if {HasFeature SymbolRecord CDA} andthen {HasFeature SymbolRecord.CDA NameA} then 
            "["#(SymbolRecord.CDA).NameA#" dummy]"
         else
            "["#NameA#" i]"
         end
      [] 'OMV'(name: Name) then
         "["#{VS2A Name}#" i]"
      [] 'OMI'(I) then
         "["#I#" num]"
      [] 'OMF'(F) then
         Num = if ({Ceil F} == F) then
                  {FloatToInt F}
               else F end
      in
         "["#Num#" num]"
      [] 'OMA'(Functor Args) then
         "("#{OM2TermPrinter Functor}
         #" "
         #if {IsList Args}
          then {RecList2TermPrinter Args}
          else {RecList2TermPrinter {Record.toList Args}}
          end
         #")"
      [] 'OMBIND'(OMS OMBVARs Term) then
         case OMS of !LAMBDA then
            {Lambda2TermPrinter OMBVARs {OM2TermPrinter Term}}
         [] !ALL_TYPES then
            %% skip all-types/kappa for now
            {OM2TermPrinter Term}
         else
            "("#{OM2TermPrinter OMS}#
            " "#{Lambda2TermPrinter OMBVARs {OM2TermPrinter Term}}#")"
         end
         /* previous version (for reference):
      [] 'OMBIND'(OMS OMBVARs Term) then
         "("#{OM2TermPrinter OMS}
         #" (lam "
         #{OM2TermPrinter OMBVARs}
         #" "      %% ToDo: Guess the right type. This is non-trivial!
         #{OM2TermPrinter Term}
         #"))"
         */
      [] 'OMBVAR'(Var) then
         {OM2TermPrinter Var}
      end
   end
   

          
end %define
   



% (:name CS-DUC-RELNS
% :type GWFF
% :context UNCLASSIFIED
% :keywords (WITH-EQUALITY WITH-DEFN PROVEN AUTO-PROOF HIGHER-ORDER SK-HIGHER-ORDER)
% :provability "Automatic expansion proof and translation to natural deduction"
% :proof-date "Monday, February 28, 2000 at 17:42:03."
% :description "CLOS-SYS2(O(O(OBC))).LAMBDA R(OBC) FORALL c(C) S-DUC(O(OB)(BBB)B) 0(B) P(BBB).R(OBC) c(C)"
% :needed-objects   (CLOS-SYS2 S-DUC)
% :mhelp "   Given a pairing algebra B (with zero element 0(B) and pairing operation P(BBB)),
% we can define a notion of join and a notion of
% inclusion.  A subset of the pairing algebra is a DUC-set (downward union closed)
% if it is downward closed with respect to inclusion, and closed with respect to
% joins.  A relation R between any set C and the pairing algebra is DUC-valued if
% for any c in C, {y | R(c,y)} is a DUC-set.  The theorem CS-DUC-RELNS states that
% the DUC-valued relations form a closure system, i.e., 
% an arbitrary intersection of DUC-valued relations is a DUC-valued relation.

% CS-DUC-RELNS: CLOS-SYS2(O(O(OBC))).LAMBDA R(OBC) FORALL c(C) S-DUC(O(OB)(BBB)B) 0(B) P(BBB).R c

%    To prove the theorem, the definitions of join and inclusion are irrelevant
% (they do not need to be instantiated).  Even without instantiating these definitions,
% the search space is quite large, as many literals have flexible heads, allowing
% them to unify with any other literal.  TPS finds a proof after about 6 hrs."
% :other-attributes NIL
% :other-remarks "
% ")


% (:name STRANGE-HO-EXAMPLE
% :type GWFF
% :context UNCLASSIFIED
% :keywords (WITHOUT-EQUALITY WITH-DEFN PROVEN AUTO-PROOF HIGHER-ORDER SK-HIGHER-ORDER)
% :provability "Automatic expansion proof and translation to natural deduction"
% :proof-date "Friday, October 13, 2000 at 15:10:21."
% :description "S(O(OII)) [STRANGE-HO-ABBR(OII(O(OII))) S(O(OII))] AND STRANGE-HO-ABBR(OII(O(OII))) S(O(OII)) x(I) y(I) IMPLIES STRANGE-HO-ABBR(OII(O(OII))) S(O(OII)) y(I) x(I)"
% :needed-objects   (STRANGE-HO-ABBR)
% :mhelp ""
% :other-attributes NIL
% :other-remarks "
% ")


