%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% XsltClient.oz ; This file is part of the MathWeb system
%
%   major updates: 19.4.2002
%
%   Authors: Andreas Franke
%
%   email: afranke@ags.uni-sb.de
%   
% For information about this program, write to:                          
%   MathWeb-Project                                                        
%   AG Siekmann/FB nformatik                                            
%   Universitaet des Saarlandes                                          
%   Bau 36, 4. Stock                                                     
%   D-66041 Saarbruecken                                                 
%   Germany    
%
% For information about the newest version of MathWeb, see 
%   http://www.mathweb.org/mathweb
%
% See the file "LICENSE" or
%    http://www.mathweb.org/mathweb/LICENSE.html
% for information on usage and redistribution
% of this file, and for a DISCLAIMER OF ALL
% WARRANTIES.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% XSLT client library
%%

functor
   
import
   OS(getEnv) Type
   HttpClient at 'x-ozlib://mathweb/httpd/HttpClient.ozf'
%  XURL at 'x-ozlib://mathweb/share/XURL.ozf'
   Browser(browse: Browse)
   System
export
   omobjToPost: OMObjToPost
   omdoc2post:  OMDoc2Post
   omdoc2TPTP:  OMDoc2TPTP
   omdoc2TPS:  OMDoc2TPS
   
define

   TOMCAT_HOST = {OS.getEnv 'TOMCAT_HOST'}
   TOMCAT_PORT = {OS.getEnv 'TOMCAT_PORT'}

   TomcatHost = if TOMCAT_HOST\=false then TOMCAT_HOST
                else "tomcat.mathweb.org" end
   TomcatPort = if TOMCAT_PORT\=false then TOMCAT_PORT
                else "8080" end

   Tomcat    = "http://"#TomcatHost#":"#TomcatPort   
   Load      = Tomcat#"/xslT/do?cmd=load"
   Transform = Tomcat#"/xslT/do?cmd=transform"

   OMDOC2TPTP = case {OS.getEnv 'OMDOC2TPTP'} of false then
                   "http://www.mathweb.org/omdoc/xsl/omdoc2tptp.xsl"
                [] FILE then FILE end
   
   OMDOC2TPS  = case {OS.getEnv 'OMDOC2TPS'} of false then
%                  "http://www.mathweb.org/omdoc/xsl/omdoc2tps.xsl"
                   "http://www.mathweb.org/~jzimmer/omdoc2tps.xsl"
                [] FILE then FILE end

   OMDOC2POST = case {OS.getEnv 'OMDOC2POST'} of false then
                   "http://www.mathweb.org/omdoc/xsl/omdoc2post.xsl"
                [] FILE then FILE end
   
   PreludeUrls =
   [
    Load#"&name=omdoc2tptp&in="#OMDOC2TPTP
    Load#"&name=omdoc2tps&in="#OMDOC2TPS
    Load#"&name=omdoc2post&in="#OMDOC2POST
   ]

   {ForAll PreludeUrls
    proc {$ Url}
       X = {HttpClient.post request(url:Url content:nil)}
    in
       {Wait X}
       {Browse a(X)}
       {System.showInfo {CondSelect X content nil}}
    end}
   
   fun {DoTransform XmlDocument StyleSheetName}
      {Type.ask.virtualString XmlDocument}
      Url = Transform#"&name="#StyleSheetName
      Response = {HttpClient.post request(url:Url content:XmlDocument)}
   in
      if (Response.statusCode div 100)==2 then
         Response.content
      else
         raise Response end
      end
   end
   
   fun {OMDoc2Post Obj} {DoTransform Obj "omdoc2post"} end
   fun {OMDoc2TPS Obj} {DoTransform Obj "omdoc2tps"} end
   fun {OMDoc2TPTP Obj} {DoTransform Obj "omdoc2tps"} end %% XXX why tps???
   
   fun {OMObjToPost OMObj} {DoTransform OMObj "omobj2post.xsl"} end % why .xsl?

end
